// $Id: Cal.java,v 1.8 1996/11/26 02:50:22 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.applet.Applet;

import CalInNet;
import CalOutNet;
import Parse;

//***************************************************************************

public class Cal extends Applet {

  static public final String idversion = "0.4.9 alpha";
  static public final String idstring = "CaL 0.4";

  boolean started;
  static boolean isApplet = true;

  Buffer tonet;
  Buffer fromnet;
  Buffer tofirstwindow;

  Socket network;

  CalAbort  abort;
  CalStatus status;

  // Host information to connect to
  // String defaulthost = "lily.acm.rpi.edu";
  String defaulthost = "eclipse.its.rpi.edu";
  // String defaulthost = "128.113.24.33";
  int    defaultport = 7777;
  String host = null;
  int    port = 0;

  CalOutNet netout;
  CalInNet  netin;
  Parse     parser;
  CalSystem system;

//---------------------------------------------------------------------------

  public void init(){
    started = false;

    //String LOGO = "connect-small.gif";
    String LOGO = "cs.gif";

    // Set some default values
    // @@@ Should be read from file
    CalValue.put( "DEFAULT", "idstring",  idstring  );
    CalValue.put( "DEFAULT", "idversion", idversion );
    CalValue.put( "DEFAULT", "platform", "JDK1.0" );
    if( isApplet ){
      CalValue.put( "DEFAULT", "host", getCodeBase().getHost() );
    } else {
      CalValue.put( "DEFAULT", "host", defaulthost );
    }
    CalValue.putInt( "DEFAULT", "port", defaultport );
    CalValue.putBool( "DEFAULT", "pause", true );
    CalValue.putBool( "DEFAULT", "review", true );

    // Need to do this processing here so we can draw something different
    if( isApplet ){
      CalValue.put( "SYSTEM", "simple", getParameter( "simple" ) );
    }

    status = new CalStatus();
    status.add( "Starting Cafe au Lys" );

    boolean issimple = false;
    try{
      issimple = CalValue.getBool( "SYSTEM", "simple" );
    } catch( CalUnknownValueException cuve ){
    }

    if( !issimple ){
      // This is an advanced client
      // The simple client does nothing for now

      // Draw the graphic
      Image image = null;
      if( isApplet ){
        URL docbase = getDocumentBase();
        image = getImage( docbase, LOGO );
      } else {
        image = Toolkit.getDefaultToolkit().getImage( LOGO );
      }
      CalGraphic graphic = new CalGraphic( image );
  
      abort = new CalAbort();
      Panel statusbar = new Panel();
      statusbar.setLayout( new BorderLayout() );
      statusbar.add( "Center", status );
      statusbar.add( "West", abort );
  
      setLayout( new BorderLayout() );
      add( "Center", graphic );
      add( "South", statusbar );
      show();
      paint( getGraphics() );
    }
  
  }

//---------------------------------------------------------------------------

  public void start(){

    if( !started ){

      if( isApplet ){
        CalValue.put( "SYSTEM", "apptype", "applet" );

        CalValue.put( "GLOBAL", "host", getParameter( "host" ) );
        CalValue.put( "GLOBAL", "port", getParameter( "port" ) );
        CalValue.put( "GLOBAL", "review", getParameter( "review" ) );
        CalValue.put( "SYSTEM", "platform", getParameter( "platform" ) );
        CalValue.put( "SYSTEM", "os", getParameter( "os" ) );

	CalValue.put( "GLOBAL", "fontsize", getParameter( "fontsize" ) );

        CalValue.put( "GLOBAL", "userid",    getParameter( "userid"    ) );
        CalValue.put( "GLOBAL", "userpass",  getParameter( "userpass"  ) );
        CalValue.put( "GLOBAL", "username",  getParameter( "username"  ) );
        CalValue.put( "GLOBAL", "userblurb", getParameter( "userblurb" ) );

      } else {
        CalValue.put( "SYSTEM", "apptype", "application" );
      }

      CalValue.put( "SYSTEM", "status", "starting" );

      system = new CalSystem( status, this );
      system.start();

      try{
        host = CalValue.get( "SYSTEM", "host" );
      } catch( CalUnknownValueException cuve ){
        status.add( "Could not find any value for host!" );
        host = defaulthost;
      }

      try{
        port = CalValue.getInt( "SYSTEM", "port" );
      } catch( CalUnknownValueException cuve ){
        status.add( "Could not find any value for port!" );
        port = defaultport;
      }

      status.add( "Connecting to " + host + ":" + port + "..." );
 
      try {
        network = new Socket( host, port );
        status.add( "Connected to " + host + ":" + port + "." );
        tonet   = new Buffer();
        fromnet = new Buffer();
  
        netout = new CalOutNet( status, tonet, network );
        netin  = new CalInNet( status, fromnet, network );

	parser = new Parse( this, status, fromnet, tonet );

        // Start the threads
        netout.start();
        netin.start();
	parser.start();

        started = true;
        CalValue.put( "SYSTEM", "status", "running" );

      } catch( UnknownHostException uhe ){
        status.add( "Unknown host " + host );
      } catch( IOException ioe ){
        status.add( "IO exception connecting to " + host );
      }

      try{
        system.join();   // Don't close up till the system thread is done
      } catch( InterruptedException ie ){
      }
  
    }

    stop();

  }  

//---------------------------------------------------------------------------

//  public void close_up(){
  public void stop(){

    System.out.println( "closing up" );
//    try{
//      system.join();   // Don't close up till the system thread is done
//    } catch( InterruptedException ie ){
//    }

    // Turn off the "bail out" button
    abort.enable( false );

    // Stop the major threads
    parser.stop();
    netout.stop();
    netin.stop();

    // Close the net
    try{
      network.close();
    } catch( IOException e ){
      status.add( "Can't close network: " + e.getMessage() );
    }

    System.out.println( "Close_up" );
  }

//---------------------------------------------------------------------------

  public static void main(String args[]) {

    // @@@ Quick hack to support a single command line option
    int co = 0;
    try{
      while( true ){
        if( args[co].equalsIgnoreCase( "-help" ) ){
          // @@@ Do something
          System.out.println( "Lame help to date:" );
          System.out.println( "  -h hostname" );
          System.out.println( "  -p port" );
          System.out.println( "  -[no]pause" );
          System.out.println( "  -[no]review" );
          System.out.println( "  -platform platform_id" );
          System.out.println( "  -os operating_system_id" );
          System.out.println( "  -fontsize points" );
          System.out.println( "  -idstring client_id" );
        } else if( args[co].equalsIgnoreCase( "-h" ) ){
          co++;
          CalValue.put( "SYSTEM", "host", args[co] );
        } else if( args[co].equalsIgnoreCase( "-p" ) ){
          co++;
          CalValue.put( "SYSTEM", "port", args[co] );
        } else if( args[co].equalsIgnoreCase( "-pause" ) ){
          CalValue.putBool( "GLOBAL", "pause", true );
        } else if( args[co].equalsIgnoreCase( "-nopause" ) ){
          CalValue.putBool( "GLOBAL", "pause", false );
        } else if( args[co].equalsIgnoreCase( "-review" ) ){
          CalValue.putBool( "GLOBAL", "review", true );
        } else if( args[co].equalsIgnoreCase( "-noreview" ) ){
          CalValue.putBool( "GLOBAL", "review", false );
        } else if( args[co].equalsIgnoreCase( "-platform" ) ){
          co++;
          CalValue.put( "SYSTEM", "platform", args[co] );
        } else if( args[co].equalsIgnoreCase( "-os" ) ){
          co++;
          CalValue.put( "SYSTEM", "os", args[co] );
        } else if( args[co].equalsIgnoreCase( "-fontsize" ) ){
          co++;
          CalValue.put( "GLOBAL", "fontsize", args[co] );
        } else if( args[co].equalsIgnoreCase( "-idstring" ) ){
          co++;
          CalValue.put( "SYSTEM", "idstring", args[co] );
        }
        co++;
      }

    } catch( ArrayIndexOutOfBoundsException aioobe ){
    }


    Frame f = new Frame("Cafe au Lys");
    Cal program = new Cal();

    isApplet = false;

    //Font oldFont = program.getFont();
    //if (oldFont == null) {
    //  System.out.println("Eek! font is null!");
    //} else {
    //  program.setFont(new Font(oldFont.getFamily(), oldFont.getStyle(),
	   //                    oldFont.getSize()+2));
    //}

    program.init();

    f.add("Center", program);
    f.resize(500, 350);
    f.pack();
    f.show();

    program.start();
  }
//---------------------------------------------------------------------------

  public Dimension minimumSize() {
    return new Dimension(500, 350);
  }

//---------------------------------------------------------------------------

  public Dimension preferredSize() {
    return minimumSize();
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalGraphic extends Canvas implements ImageObserver{

  Image image;
  Graphics gr;

//---------------------------------------------------------------------------

  CalGraphic( Image i ){
    image = i;
  }

//---------------------------------------------------------------------------

  public boolean imageUpdate(Image img, int infoflags,
                             int x, int y, int width, int height) {

    if ((infoflags & (WIDTH | HEIGHT)) != 0) {
    }

    if ((infoflags & ALLBITS) != 0) {
      repaint();
    }

    return (infoflags & (ALLBITS | ERROR)) == 0;
  }

//---------------------------------------------------------------------------

  public void paint( Graphics g ){

    String id;
    try{
      id = "Cafe au Lys - " + CalValue.get( "SYSTEM", "idversion" );
    } catch( CalUnknownValueException cuve ){
      id = "Cafe au Lys";
    }

    // Draw the mug
    g.drawRect( 49, 150, 201, 150 );
    g.drawArc( 230, 200, 40, 40, 270, 180 );
    g.drawArc( 210, 180, 80, 80, 270, 180 );

    // The Connect logo
    gr = g;
    if( image != null ){
      g.drawImage( image, 50, 160, this );
    }
 
    // And some text
    Font f = g.getFont();
    g.setFont( new Font( f.getName(), f.getStyle(), 20 ) );
    g.drawString( id, 250, 50 );

    // Draw the lily
    g.setColor( Color.red );
    g.drawLine( 150, 10, 150, 150 );
    g.drawArc( 125, 25, 25, 50, 270, 180 );
    g.drawArc( 150, 25, 25, 50, 270, -180 );

  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalSystem implements Runnable{

  Thread thread;

  CalStatus statusbar;
  Cal applet;

//---------------------------------------------------------------------------

  CalSystem( CalStatus s, Cal a ){
    statusbar = s;
    applet = a;
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, "CalSystem" );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  public void run(){
    String oldstatus = "";
    String status;
    try{
      status = CalValue.get( "SYSTEM", "status" );
    } catch( CalUnknownValueException uve1 ){
      status = "";
    }
    
    while( !status.equalsIgnoreCase( "shutdown" ) ){
      try{
        thread.sleep( 1000 );
        status = CalValue.get( "SYSTEM", "status" );
      } catch( InterruptedException ie ){
        System.out.println( "InterruptedException in CalSystem" );
      } catch( CalUnknownValueException uve2 ){
        status = "unknown";
      }
      if( status != oldstatus ){
        statusbar.add( "Status changed to '" + status +"'" );
        oldstatus = status;
      }
    }

    try{
      if( CalValue.get( "SYSTEM", "apptype" ).equalsIgnoreCase("applet") ){
        //applet.close_up();
        statusbar.add( "You may now close this page." ); 
        //applet.destroy();   // @@@ Does this work?
      } else {
        // application
        System.exit( 0 );
      }
    } catch( CalUnknownValueException uve3 ){
      statusbar.add( "Unknown app type.  Cannot close down." );
    }

  }

//---------------------------------------------------------------------------

  public void stop(){
    System.out.println( "Stopping " + thread.getName() );
    thread.stop();
    thread = null;
    System.out.println( "Stopped" );
  }

//---------------------------------------------------------------------------

  public void join() throws InterruptedException{
    thread.join();
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

