// $Id: CalStatus.java,v 1.1 1996/11/25 14:08:18 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.util.*;

//***************************************************************************

class CalStatusLine{

  Date   time;
  String desc;

//---------------------------------------------------------------------------

  CalStatusLine( Date d, String s ){
    time = d;
    desc = s;
  }

//---------------------------------------------------------------------------

  CalStatusLine( String s ){
    this( new Date(), s );
  }

//---------------------------------------------------------------------------

  public Date getTime(){
    return time;
  }

//---------------------------------------------------------------------------

  public String getDesc(){
    return desc;
  }

//---------------------------------------------------------------------------

  public String getString(){
    return( '(' + time.toLocaleString() + ") - " + desc );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalStatus extends Panel{

  TextField field;
  Vector events;

//---------------------------------------------------------------------------

  CalStatus(){
    field = new TextField();
    field.setEditable( false );
    setLayout( new BorderLayout() );
    add( "Center", field );
    add( "East", new CalButton( "Status" ) );

    events = new Vector();
  }

//---------------------------------------------------------------------------

  public void add( String message ){
    field.setText( message );
    events.addElement( new CalStatusLine( message ) );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event e ){
    if( CalButton.buttonHit( e ) ){

      // This is a status notice
      add( "Opening CaL Status window" );

      // Create a window showing events to date
      new CalStatusWindow( this, events );
      
      return true;
    } 
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalStatusWindow extends Frame{

  CalStatus status;
  TextArea area;

//---------------------------------------------------------------------------

  CalStatusWindow( CalStatus s, Vector v ){

    status = s;

    area = new TextArea();
    area.setEditable( false );

    setLayout( new BorderLayout() );
    add( "Center", area );
    add( "South", new CalButton( "Close" ) );
    pack();
    show();

    CalStatusLine stat_line;
    String line;
    int co = 0;
    try{
      while( true ){
        stat_line = (CalStatusLine)v.elementAt( co );
        line = stat_line.getString();
        area.appendText( line + '\n' );
        co++;
      }
    } catch( ArrayIndexOutOfBoundsException e ){
      // Normal - we're out of the loop
    }
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event e ){
    if( CalButton.buttonHit( e ) ){

      // Close ourselves
      dispose();

      // This is a status notice
      status.add( "Closing CaL Status window" );

      return true;
    } 
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalAbort extends CalButton{

//---------------------------------------------------------------------------

  CalAbort(){
    super( "Bail Out" );
  }

//---------------------------------------------------------------------------

  public boolean hit( Event e ){
    // Set our terminate status
    CalValue.put( "SYSTEM", "status", "shutdown" );

    // Get out
    return true;
  }

//---------------------------------------------------------------------------

}

