// $Id: CalValue.java,v 1.2 1996/10/17 00:52:31 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.util.*;

//***************************************************************************

class CalUnknownValueException extends RuntimeException{

//---------------------------------------------------------------------------

  CalUnknownValueException(){
    super();
  }

//---------------------------------------------------------------------------

  CalUnknownValueException( String s ){
    super( s );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalValue{

//---------------------------------------------------------------------------

  static private Hashtable values;

//---------------------------------------------------------------------------

  static void init(){
    // Create the master hashtable and some basic values
    values = new Hashtable();
    Hashtable t1 = new Hashtable();
    Hashtable t2 = new Hashtable();
    values.put( "DEFAULT", t1 );
    values.put( "GLOBAL", t2 );
    t1.put( "NAME", "DEFAULT" );
    t2.put( "NAME", "GLOBAL" );
  }

//---------------------------------------------------------------------------

  static public synchronized String get( String table, String name )
    throws CalUnknownValueException{

    String ret = null;

    // Create the master hashtable, if necessary
    if( values == null ){
      init();
    }

    Hashtable t;

    // Try it in the requested table first
    t = (Hashtable)values.get( table );
    if( t != null ){
      ret = (String)t.get( name );
    }

    if( ret == null ){
      // Try it in the GLOBAL table next
      t = (Hashtable)values.get( "GLOBAL" );
      if( t != null ){
        ret = (String)t.get( name );
      }
    }

    if( ret == null ){
      // Finally try the DEFAULT table
      t = (Hashtable)values.get( "DEFAULT" );
      if( t != null ){
        ret = (String)t.get( name );
      }
    }

    if( ret == null ){
      throw new CalUnknownValueException( table + "." + name );
    }

    return ret;
  }

//---------------------------------------------------------------------------

  static public int getInt( String table, String name )
    throws CalUnknownValueException{

    int ret = 0;
    String value = get( table, name );

    try{
      ret = Integer.parseInt( value, 10 );
    } catch( NumberFormatException nfe ){
      throw new CalUnknownValueException( table + "." + name );
    }
    
    return ret;
  }

//---------------------------------------------------------------------------

  static public boolean getBool( String table, String name )
    throws CalUnknownValueException{

    boolean ret = false;
    String value = get( table, name );

    if( value.equalsIgnoreCase( "true" ) ){
      ret = true;
    } else if( value.equalsIgnoreCase( "yes" ) ){
      ret = true;
    } else if( value.equalsIgnoreCase( "on" ) ){
      ret = true;
    } else if( value.equalsIgnoreCase( "false" ) ){
      ret = false;
    } else if( value.equalsIgnoreCase( "no" ) ){
      ret = false;
    } else if( value.equalsIgnoreCase( "off" ) ){
      ret = false;
    } else {
      throw new CalUnknownValueException( table + "." + name );
    }

    return ret;
  }

//---------------------------------------------------------------------------

  static public synchronized void put( String table,String name,String value ){

    // Create the master hashtable, if necessary
    if( values == null ){
      init();
    }

    // First, get the table or create a new one
    Hashtable t = (Hashtable)values.get( table );
    if( t == null ){
      t = new Hashtable();
      values.put( table, t );
      t.put( "NAME", table );
    }

    // Now put the value there
    try{
      t.put( name, value );
    } catch( NullPointerException nve ){
      System.err.println( "NullPointerException in CalValue.put()" );
    }
  }

//---------------------------------------------------------------------------

  static public void putInt( String table,String name,int value ){
    put( table, name, Integer.toString( value ) );
  }

//---------------------------------------------------------------------------

  static public void putBool( String table,String name,boolean value ){
    if( value ){
      put( table, name, "true" );
    } else {
      put( table, name, "false" );
    }
  }
         
//---------------------------------------------------------------------------

  static public void dump(){
    
    Enumeration tables;
    Enumeration names;
    Hashtable   nametable;
    String tab;
    String attribute;
    String value;

    tables = values.keys();
    while( tables.hasMoreElements() ){
      tab = (String)(tables.nextElement());
      System.out.println( "\nTable: " + tab );
      nametable = (Hashtable)(values.get( tab ));
      names = nametable.keys();
      while( names.hasMoreElements() ){
        attribute = (String)(names.nextElement());
        value = (String)(nametable.get( attribute ));
        System.out.println( attribute + '=' + value );
      }

    }

  }

//---------------------------------------------------------------------------

  public static void main( String args[] ){

    CalValue.put( "DEFAULT", "aa", "bb" );
    CalValue.put( "DEFAULT", "a", "b" );
    CalValue.put( "foo", "a", "c" );
    System.out.println( CalValue.get( "GLOBAL", "a" ) );
    System.out.println( CalValue.get( "foo", "a" ) );
    System.out.println( CalValue.get( "bar", "a" ) );
    System.out.println( CalValue.get( "bar", "aa" ) );

    CalValue.put( "GLOBAL", "aa", "cc" );
    System.out.println( CalValue.get( "bar", "aa" ) );

    CalValue.dump();
  }

//---------------------------------------------------------------------------

}

