// $Id: Menu.java,v 1.1 1996/05/30 01:26:39 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.util.*;

import CalWindow;

//***************************************************************************

class CalMenu extends MenuBar{

//---------------------------------------------------------------------------

  CalMenu( CalWindow parent ){
    super();

    add( new MenuFile( parent ) );
    add( new MenuList( parent ) );
    add( new MenuWindow( parent ) );

    MenuHelp help = new MenuHelp( parent );
    add( help );
    setHelpMenu( help );
  }

//---------------------------------------------------------------------------

  public boolean postEvent( Event evt ){
    if( evt.target instanceof MenuItemAction ){
      return ((MenuItemAction)evt.target).action( evt, new Object() );
      // @@@ new Object   is a hack!
    }

    return super.postEvent( evt );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

interface MenuItemAction{
  public abstract boolean action( Event evt, Object obj );
}

//***************************************************************************

class MenuList extends Menu{

  public static String LIST = "list...";

//---------------------------------------------------------------------------

  MenuList( CalWindow parent ){
    super( "List", true );    // Allow tear-offs

    Buffer      parser  = parent.getParseBuffer();

    Menu whoMenu = new Menu( "Who" );    
    whoMenu.add( new MenuCalCommand( "Here", "wholist here", parser ) );
    whoMenu.add( new MenuCalCommand( "Away", "wholist away", parser ) );
    whoMenu.add( new MenuCalCommand( "Detached", "wholist detached", parser ) );
    whoMenu.add( new MenuCalCommand( "All", "wholist all", parser ) );
    whoMenu.add( new MenuCalCommand( "Everyone", "wholist everyone", parser ) );
    whoMenu.addSeparator();
    whoMenu.add( new MenuCalCommand( LIST, "wholist " + LIST, parser ) );

    Menu whatMenu = new Menu( "What" );
    whatMenu.add( new MenuCalCommand( "Active", "whatlist active", parser ) );
    whatMenu.add( new MenuCalCommand( "Inactive", "whatlist inactive", parser ) );
    whatMenu.add( new MenuCalCommand( "All", "whatlist all", parser ) );
    whatMenu.add( new MenuCalCommand( "My discussions", "whatlist mine", parser ) );
    whatMenu.addSeparator();
    whatMenu.add( new MenuCalCommand( LIST, "whatlist " + LIST, parser ) );

    Menu oldmsgsMenu = new Menu( "Messages" );
    oldmsgsMenu.add( new MenuCommand( "Detached", "/REVIEW DETACH", parser ) );
    oldmsgsMenu.add( new MenuCommand( "Old", "/REVIEW OLD", parser ) );

    add( whoMenu );
    add( whatMenu );
    addSeparator();
    add( oldmsgsMenu );
    addSeparator();
    add( new MenuCalCommand( "Select buffer", "select", parser ) );
    
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuWindow extends Menu{

//---------------------------------------------------------------------------

  MenuWindow( CalWindow parent ){
    super( "Options", true );    // Allow tear-offs

    Buffer      network = parent.getNetworkBuffer();
    String      name    = null;
    Buffer      parser  = parent.getParseBuffer();

    LineCounter lines   = null;
    CalArea     area    = null;
    
    if( parent instanceof CalTextWindow ){
      lines = ((CalTextWindow)parent).getLines();
      area  = ((CalTextWindow)parent).getArea();
      name  = parent.getName();
    }

    add( new MenuWindowPause( name ) );
    add( new MenuWindowResetpauselines( lines, area ) );
    addSeparator();
    add( new MenuCalCommand( "Info file", "infoedit me", parser ) );    
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuFile extends Menu{

//---------------------------------------------------------------------------

  MenuFile( CalWindow parent ){
    super( "Window", true );    // Allow tear-offs
    
    Buffer      parser  = parent.getParseBuffer();
    String      name    = parent.getName();

    add( new MenuFileLog() );
    addSeparator();
    add( new MenuWindowClose( name, parser ) );
    add( new MenuFileQuit( parser ) );
    
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuHelp extends Menu{

//---------------------------------------------------------------------------

  MenuHelp( CalWindow parent ){
    super( "Help", true );    // Allow tear-offs
    
    Buffer      parser  = parent.getParseBuffer();

    add( new MenuHelpNewuser() );
    add( new MenuHelpSubmit() );
    addSeparator();
    add( new MenuCommand( "About me...", "/ME", parser ) );
    add( new MenuHelpAboutCal() );
    add( new MenuHelpAboutLily() );
    
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuFileLog extends CheckboxMenuItem implements MenuItemAction{

//---------------------------------------------------------------------------

  MenuFileLog(){
    super( "Log..." );
    enable( false );       // @@@ Implement this function
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuWindowPause extends CheckboxMenuItem implements MenuItemAction{

  LineCounter lines;
  String buffname;

//---------------------------------------------------------------------------

  MenuWindowPause( String n ){
    super( "Pause" );
    buffname = n;

    if( n != null ){

      // Do we start out pausing?
      boolean state;
      try{
        state = CalValue.getBool( buffname, "pause" );
      } catch( CalUnknownValueException cuve ){
        state = true;
      }
      setState( state );

      enable( true );
    } else {
      enable( false );
    }
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    boolean state = getState();
    CalValue.putBool( buffname, "pause", state );
    return true;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuWindowResetpauselines extends MenuItem implements MenuItemAction{

  LineCounter lines;
  CalArea area;

//---------------------------------------------------------------------------

  MenuWindowResetpauselines( LineCounter l, CalArea a ){
    super( "Pause reset" );
    lines = l;
    area = a;
    if( (l != null) && (a != null) ){
      enable( true );
    } else {
      enable( false );
    }
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    area.charLines();
    return true;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuFileQuit extends MenuItem implements MenuItemAction{

  Buffer parser;

//---------------------------------------------------------------------------

  MenuFileQuit( Buffer p ){
    super( "Quit..." );
    enable( true );
    parser = p;
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    parser.putOOB( "%cal quit\n" );
    return true;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuWindowClose extends MenuItem implements MenuItemAction{

  Buffer parser;
  String window_name;

//---------------------------------------------------------------------------

  MenuWindowClose( String wn, Buffer p ){
    super( "Close" );
    if( wn.equals( Parse.DEFAULT ) ){
      // Do not allow the default window to be destroyed
      enable( false );
    } else {
      enable( true );
    }
    window_name = wn;
    parser = p;
  }

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    
    parser.putOOB( "%cal close " + window_name + '\n' );
    return true;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuHelpNewuser extends MenuItem implements MenuItemAction{

//---------------------------------------------------------------------------

  MenuHelpNewuser(){
    super( "New Users..." );
    enable( false );       // @@@ Implement this function
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuHelpSubmit extends MenuItem implements MenuItemAction{

//---------------------------------------------------------------------------

  MenuHelpSubmit(){
    super( "Submit a Bug Report" );
    enable( false );       // @@@ Implement this function
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuHelpAboutCal extends MenuItem implements MenuItemAction{

//---------------------------------------------------------------------------

  MenuHelpAboutCal(){
    super( "About CaL..." );
    enable( false );       // @@@ Implement this function
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuHelpAboutLily extends MenuItem implements MenuItemAction{

//---------------------------------------------------------------------------

  MenuHelpAboutLily(){
    super( "About lily..." );
    enable( false );       // @@@ Implement this function
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuCommand extends MenuItem implements MenuItemAction{

  Buffer toparser;
  String command = null;

//---------------------------------------------------------------------------

  MenuCommand( String l, String c, Buffer tp ){
    super( l );
    toparser = tp;
    if( c.length() > 0 ){
      command = "%cal command " + c + '\n';
      enable( true );
    } else {
      enable( false );
    }
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    if( command != null ){
      toparser.putOOB( command );
      return true;
    } else {
      return false;
    }
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class MenuCalCommand extends MenuItem implements MenuItemAction{

  Buffer toparser;
  String command = null;

//---------------------------------------------------------------------------

  MenuCalCommand( String l, String c, Buffer tp ){
    super( l );
    toparser = tp;
    if( c.length() > 0 ){
      command = "%cal " + c + '\n';
      enable( true );
    } else {
      enable( false );
    }
  }  

//---------------------------------------------------------------------------

  public boolean action( Event evt, Object obj ){
    if( command != null ){
      toparser.putOOB( command );
      return true;
    } else {
      return false;
    }
  }

//---------------------------------------------------------------------------

}

//***************************************************************************
