// $Id: Net.java,v 1.1 1996/05/30 01:26:56 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.util.*;
import java.net.*;
import java.io.*;

//***************************************************************************

class CalOutNet implements Runnable{

  Thread thread;

  CalStatus status;
  Buffer buffer;
  OutputStream os;

//---------------------------------------------------------------------------

  CalOutNet( CalStatus s, Buffer b, Socket n ){
    status = s;
    buffer = b;
    try{
      os = n.getOutputStream();
    } catch( IOException e ){
      status.add( "IOException: " + e.getMessage() );
    }
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, "OutNet" );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  public void run(){
    boolean connected= true;

    while( connected ){
      String text = buffer.get() + "\n";   // Why do we have to add this?
      int buffLen = text.length();
      byte buff[] = new byte[ buffLen ];
      text.getBytes( 0, buffLen - 1, buff, 0 );

      try{
        os.write( buff );
	os.flush();
      } catch( IOException e ){
        status.add( "IOException: " + e.getMessage() );
        connected = false; 
      }
      
    }
  }

//---------------------------------------------------------------------------

  public void stop(){
    System.out.println( "CalOutNet stop" );
    if( thread != null ){
      thread.stop();
      thread = null;
    }
    System.out.println( "stopped" );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalInNet implements Runnable{

  Thread thread;

  CalStatus status;
  Buffer buffer;
  InputStream is;

//---------------------------------------------------------------------------

  CalInNet( CalStatus s, Buffer b, Socket n ){
    status = s;
    buffer = b;
    try{
      is = n.getInputStream();
    } catch( IOException e ){
      status.add( "IOException: " + e.getMessage() );
    }
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, "InNet" );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  public void run(){
    int bufflen = 512;
    byte buff[] = new byte[ bufflen ];
    int readsize;

    boolean connected = true;

    while( connected ){

      try{
        readsize = is.read( buff, 0, bufflen );   // Blocks
	if( readsize > 0 ){
          String readstring = new String( buff, 0, 0, readsize );
          buffer.put( readstring );
        }
      } catch( IOException e ){
        status.add( "IOException: " + e.getMessage() );
        connected = false;
      }

    }
  }

//---------------------------------------------------------------------------

  public void stop(){
    System.out.println( "CalInNet stop" );
    if( thread != null ){
      thread.stop();
      thread = null;
    }
    System.out.println( "stopped" );
  }

//---------------------------------------------------------------------------

}


