// $Id: Parse.java,v 1.4 1996/11/25 01:15:38 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.util.*;
import java.applet.Applet;

import CalTextWindow;
import Buffer;
import QuitWindow;

//***************************************************************************

class CalUnknownBuffException extends RuntimeException{

//---------------------------------------------------------------------------

  CalUnknownBuffException(){
    super();
  }

//---------------------------------------------------------------------------

  CalUnknownBuffException( String s ){
    super( s );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class ParseLine{

  String id = null;
  String line = null;

//---------------------------------------------------------------------------

  ParseLine( String l ){
    if( l.indexOf( "%command" ) == 0 ){
      // Remove the %command and get the task id and the rest of the
      // line
      String params[] = CalUtil.split( l );
      id = params[1];
      int lastbrace = l.indexOf( "]" );
      try{
        line = l.substring( lastbrace+2 );
      } catch( StringIndexOutOfBoundsException e ){
      }

      System.out.println( "COMMAND '" + line + "'" );
    } else {
      line = l;
    }
  }

//---------------------------------------------------------------------------

  public String taskId(){
    // Get just the task ID string
    return id;
  }

//---------------------------------------------------------------------------

  public String text(){
    // Get the rest of the line
    return line;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class Parse implements Runnable{

  Thread thread;

  CalStatus status;

  Buffer fromnet;  // What we get from the server
  Buffer tonet;    // If we have to talk to the server

  BufferTable buffers;
  static final String DEFAULT = "Default";
  String redirect = null;            // Where to redirect output to

  Hashtable cmdbuffer;  // Map a command to a buffer
  Hashtable idbuffer;   // Map a taskid to a buffer
  Vector    directable; // Buffers which want to see a %begin/%end tag

  boolean logged_in = false;
  boolean reviewing = false;
  boolean connected = false;
  boolean prompted_for_password = false;
  boolean prompted_for_name     = false;
  String uls = ""; // Unknown login stuff    @@@

  //final String idString = "lily 0.8b";
  //final String idString = "CaL 0.4";
  String idString;

  Applet applet;

//---------------------------------------------------------------------------

  Parse( Applet app, CalStatus s, Buffer from, Buffer to ){
    applet = app;
    status = s;
    fromnet = from;
    tonet   = to;

    buffers    = new BufferTable();
    cmdbuffer  = new Hashtable();
    idbuffer   = new Hashtable();
    directable = new Vector();

    try{
      idString = CalValue.get( "SYSTEM", "idstring" );
    } catch( CalUnknownValueException cuve ){
      idString = "CaL ?";
    }
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, "Parser" );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  String detached = "do you wish to review";

  public void run(){
    String text;

    while( true ){
      text = fromnet.get();

      if( !logged_in ){
        status.add( "Logging in" );
        text = parse_login( text );
      } 

      if( logged_in ){
        if( !connected ){

          if( text.indexOf( detached ) != -1 ){
            status.add( "Attaching" );
            tonet.put( "n\n" );   // Do not ask for the review
          }

          if( text.indexOf( "%connected" ) != -1 ){
            connected = true;
            fromnet.setLinemode( true );
            status.add( "Fully connected" );

            // Bring up the initial frame
            Buffer defbuffer = new Buffer();
            boolean issimple = false;
            try{
              issimple = CalValue.getBool( "SYSTEM", "simple" );
            } catch( CalUnknownValueException cuve ){
            }
            if( issimple ){
              // In the simple world, we squeeze it into the
              // browser window
              applet.add( new LoCalTextWindow( DEFAULT, status, 
                                               defbuffer, tonet, fromnet ) );
              applet.show();
              applet.validate();
            } else {
              // Complex systems bring up a new window
              CalTextWindow defaultwindow = 
                new CalTextWindow( DEFAULT, status, defbuffer, tonet,
                                   fromnet );
            }
            defbuffer.setLinemode( true );
            buffers.put( DEFAULT, defbuffer );

            // Parse the rest of the text
            System.out.println( "Remaining text: '" + text + "'" );
            String leftover[] = CalUtil.split( text, "\n" );
            int co = 0;
            try{
              while( true ){
                parse_normal( leftover[co] + '\n' );
                co++;
              }
            } catch( ArrayIndexOutOfBoundsException aioobe ){
            }

          }
        } else {
          // We are logged in and connected
          parse_normal( text );
        }
      }
    }
  }

//---------------------------------------------------------------------------

  void parse_normal( String line ){
    ParseLine pl  = new ParseLine( line );
    String taskid = pl.taskId();
    String text   = pl.text();
    String buffer;
    try{
      buffer = (String)idbuffer.get( taskid );
    } catch( NullPointerException npe ){
      buffer = DEFAULT;
    }

    if( text.indexOf( "%" ) == 0 ){
      parse_percent( text );
    } else if( text.indexOf( "(" ) == 0 ){
      parse_status( text, buffer );
    } else {
      // When there is noplace else to go...
      default_send( text, buffer );
    }

  }

//---------------------------------------------------------------------------

  void default_send( String text ){
    default_send( text, null );
  }

//---------------------------------------------------------------------------

  void default_send( String text, String buffer ){

    String name;
    if( redirect != null ){
      name = redirect;
    } else {
      if( buffer != null ){
        name = buffer;
      } else {
        name = DEFAULT;
      }
    }

    try{
      send_to_buffer( name, text );
    } catch( CalUnknownBuffException e ){
      status.add( "Unknown Buffer: " + e.getMessage() );
    }
  }

//---------------------------------------------------------------------------

  void oob_default_send( String text ){
    oob_default_send( text, null );
  }

//---------------------------------------------------------------------------

  void oob_default_send( String text, String buffer ){

    String name;
    if( redirect != null ){
      name = redirect;
    } else {
      if( buffer != null ){
        name = buffer;
      } else {
        name = DEFAULT;
      }
    }

    try{
      oob_send_to_buffer( name, text );
    } catch( CalUnknownBuffException e ){
      status.add( "Unknown Buffer: " + e.getMessage() );
    }
  }

//---------------------------------------------------------------------------

  String parse_login( String line ){

    boolean parsed = false;
    // Each if statement is seperate, since it is possible for 
    // more than one part to show up in the same packet or parsable
    // set of packets.

    System.out.println( "parse_login: '" + line + "'" );
    uls = uls + line;

    if( uls.indexOf( "login:" ) != -1 ){
      // We are being prompted for login
      if( prompted_for_password ){
        // We tried this once and it failed, so ask for it.
        LoginBox login = new LoginBox( uls, tonet );
      } else {
        try{
          String login = CalValue.get( "GLOBAL", "userid" );
          String pass  = CalValue.get( "GLOBAL", "userpass" );
          tonet.put( login + ' ' + pass + '\n' );
        } catch( CalUnknownValueException cuve1 ){
          LoginBox login = new LoginBox( uls, tonet );
        }
      }
      parsed = true;
      prompted_for_password = true;
    } 

    if( uls.indexOf( "*** Timed-out waiting for login. ***" ) != -1 ){
      // Close it up if we lost the connection
      CalValue.put( "SYSTEM", "status", "shutdown" );
    }

    if( (uls.indexOf("*** Connected ***") != -1) ||
        (uls.indexOf("*** Redirecting old connection to this port ***") != -1)
      ){
      // Send the client ID once connected
      tonet.put( "#$# client " + idString + '\n' );
      parsed = true;
    } 

    if( (uls.indexOf( "Please choose a name" ) != -1) &&
        (uls.indexOf( "-->" ) != -1 ) ){
      if( prompted_for_name ){
        // If we have already been prompted once, we have to ask
        NameBox name = new NameBox( uls, tonet );
      } else {
        try{
          String namestring = CalValue.get( "GLOBAL", "username" );
          tonet.put( namestring + '\n' );
        } catch( CalUnknownValueException cuve2 ){
          NameBox name = new NameBox( uls, tonet );
        }
      }
      parsed = true;
      prompted_for_name = true;
    }

    if( (uls.indexOf( "blurb" ) != -1) &&
        (uls.indexOf( "-->" ) != -1 ) ){
      try{
        // Only deal with blurbs on advanced clients
        if( !CalValue.getBool( "SYSTEM", "simple" ) ){
          String blurbstring = CalValue.get( "GLOBAL", "userblurb" );
          tonet.put( blurbstring + '\n' );
        } else {
          tonet.put( "\n" );
        }
      } catch( CalUnknownValueException cuve3 ){
        BlurbBox blurb = new BlurbBox( uls, tonet );
      }
      parsed = true;
    }

    if( uls.indexOf( "type /HELP for an introduction" ) != -1 ){
      // @@@ Need to find a better marker, but see the connected
      //     boolean too
      status.add( "Logged in" );
      logged_in = true;
    } 

    if( !parsed ){
      System.out.println( "Unknown login stuff: " + line );
    } else {
      // It was parsed, so clear the uls
      uls = "";
    }

    return uls;
  }

//---------------------------------------------------------------------------

  void parse_percent( String command_line ){
    String command = null;
    String parameter[];
    int num_param;

    parameter = CalUtil.split( command_line );
    command = parameter[0];

    // Now we can take action
    if( command.equalsIgnoreCase( "%cal" ) ){
      parse_percent_cal( parameter );
    } else if( command.equalsIgnoreCase( "%info_size" ) ){
      // @@@ Ignore
    } else if( command.equalsIgnoreCase( "%recip_regexp" ) ){
      // @@@ Ignore
    } else if( command.equalsIgnoreCase( "%connected" ) ){
      if( parameter[1].equalsIgnoreCase( "attach" ) ){
        try{
          if( CalValue.getBool( "Default", "review" ) ){
            fromnet.putOOB( "%cal command /REVIEW DETACH\n" );
          }
        } catch( CalUnknownValueException cuve ){
          System.out.println( "no review boolean set" );
        }
      }
    } else if( command.equalsIgnoreCase( "%sender" ) ){
      CalValue.put( "GLOBAL", "last_recip", parameter[1] );
    } else if( command.equalsIgnoreCase( "%sendgroup" ) ){
      CalValue.put( "GLOBAL", "last_group", parameter[1] );
    } else if( command.equalsIgnoreCase( "%endmsg" ) ){
      // @@@ Implement
    } else if( command.equalsIgnoreCase( "%begin" ) ){

      try{
        // Find the taskid and the command that prompted this
        int firstparen;
        int lastparen;
        int firstbracket;
        int lastbracket;

        String taskcmd;
        String taskid;

        // First try to figure out which format of %begin we're using:
        //   %begin (cmd) [taskid]     (old) or
        //   %begin [taskid] cmd       (new)
        firstparen   = command_line.indexOf( "(" );
        firstbracket = command_line.indexOf( "[" );
        if( (firstparen == -1) || (firstbracket < firstparen) ){
          // New format
          lastbracket  = command_line.indexOf( "]" ) + 1;
          taskid  = command_line.substring( firstbracket, lastbracket );
          firstparen   = lastbracket + 1;   // Not really, but the data is here
          taskcmd = '(' + command_line.substring( firstparen ).trim() + ')';
        } else {
          // Old format
          lastparen    = command_line.lastIndexOf( ")" ) + 1;
          firstbracket = command_line.lastIndexOf( "[" );
          lastbracket  = command_line.lastIndexOf( "]" ) + 1;
          taskcmd = command_line.substring( firstparen,   lastparen );
          taskid  = command_line.substring( firstbracket, lastbracket );
        }

        System.out.println( "%begin: taskcmd="+taskcmd+" taskid="+taskid );

        // Find the buffer that we should associate the task id with
        //  and then remove it from the system so we don't trip over it
        String buffer = (String)cmdbuffer.remove( taskcmd );
        if( buffer == null ){
          buffer = DEFAULT;        
        } else {
          // If there was really a buffer, check to see if it wants
          //   the begin tag
          if( directable.contains( buffer ) ){
            default_send( "%begin\n", buffer );
          }
        }

        // Associate the taskid with the buffer
        idbuffer.put( taskid, buffer );
        
      } catch( StringIndexOutOfBoundsException sioobe ){
      } catch( NullPointerException npe ){
      }

    } else if( command.equalsIgnoreCase( "%end" ) ){
      // Remove the entry from the taskid list
      String taskid = parameter[1];
      String buffer = (String)idbuffer.remove( taskid );
      if( buffer != null ){
        if( directable.contains( buffer ) ){
          default_send( "%end\n", buffer );
          directable.removeElement( buffer );
        }
      }

    } else if( command.indexOf( "%g" ) == 0 ){
      // This one should be last, since it is not checking the full
      // command word, but the first part of the line

      // Send the signal to the window to handle
      oob_default_send( "%g\n" );

      // Then send the rest of the line there
      try{
        default_send( command_line.substring( 2 ) );
      } catch( StringIndexOutOfBoundsException e ){
      }

    } else {
      status.add( "Unknown % command '" + command + "'" );
    }

  }

//---------------------------------------------------------------------------

  void parse_percent_cal( String param[] ){
    String command = param[1];   // Not 0, since thats "%cal"

    if( command.equalsIgnoreCase( "close" ) ){
      try{
        oob_send_to_buffer( param[2], "%cal close\n" );
        buffers.remove( param[2] );   // Take it out of our list
        status.add( "Removed window " + param[2] );
      } catch( CalUnknownBuffException e ){
        status.add( "Can't delete unknown buffer: " + e.getMessage() );
      }

    } else if( command.equalsIgnoreCase( "select" ) ){
      String name = buffers.uniqueName( "selected" );
      Buffer buffer = new Buffer();
      buffer.setLinemode( true );
      CalSelectWindow window = 
        new CalSelectWindow( name, status, buffer, tonet, fromnet );
      buffers.put( name, buffer );
      status.add( "Creating new selected list: " + name );
      
    } else if( command.equalsIgnoreCase( "infoedit" ) ){
      String name = buffers.uniqueName( "info " + param[2] );
      Buffer buffer = new Buffer();
      buffer.setLinemode( true );
      CalInfoWindow window = 
        new CalInfoWindow( name, status, buffer, tonet, fromnet, param[2] );
      buffers.put( name, buffer );
      status.add( "Creating new editable info buffer: " + name );

    } else if( command.equalsIgnoreCase( "wholist" ) ){
      String name = buffers.uniqueName( "who " + param[2] );
      Buffer whobuffer = new Buffer();
      whobuffer.setLinemode( true );
      CalWhoWindow whowindow = 
        new CalWhoWindow( name, status, whobuffer, tonet, fromnet, param[2] );
      buffers.put( name, whobuffer );
      status.add( "Creating new who list: " + name );

    } else if( command.equalsIgnoreCase( "whatlist" ) ){
      String name = buffers.uniqueName( "what " + param[2] );
      Buffer whatbuffer = new Buffer();
      whatbuffer.setLinemode( true );
      CalWhatWindow whatwindow = 
        new CalWhatWindow(name, status, whatbuffer, tonet, fromnet, param[2]);
      buffers.put( name, whatbuffer );
      status.add( "Creating new what list: " + name );
    } else if( command.equalsIgnoreCase( "command" ) ){
      // Create a new window and direct the output of the command
      // to there without further parsing
      String commd = CalUtil.unsplit( param, " ", 2 );
      String name = buffers.uniqueName( "Command " + commd );
      Buffer buffer = new Buffer();
      buffer.setLinemode( true );
      CalTextWindow window = 
        new CalTextWindow( name, status, buffer, tonet, fromnet );
      buffers.put( name, buffer );
      status.add( "Creating new command window: " + name );
      tonet.put( commd + '\n' );

      // Store the command and the buffer for lookup later
      try{
        cmdbuffer.put( '('+commd+')', name );
        System.out.println( "name: " + name + " commd: "+'('+commd+')' );
      } catch( NullPointerException npe ){
        System.out.println( "npe while storing command" );
      }
    } else if( command.equalsIgnoreCase( "dcommand" ) ){
      // Just direct the output elsewhere
      String commd = CalUtil.unsplit( param, " ", 3 );
      String name = param[2];
      status.add( "Directing some output to: " + name );
      tonet.put( commd + '\n' );

      // Store the command and the buffer for lookup later
      try{
        cmdbuffer.put( '('+commd+')', name );
      } catch( NullPointerException npe ){
      }

      // Mark this as one to forward the beginning and end for
      directable.addElement( name );
      
    } else if( command.equalsIgnoreCase( "quit" ) ){
      // Bring up the window to shut down CaL
      QuitWindow quitWindow = new QuitWindow( tonet, "CaL: Quit" );
    } else {
      status.add( "Unknown %cal command '" + command + "'" );
    }    
  }

//---------------------------------------------------------------------------

  void parse_status( String text, String buffer ){
    if( text.indexOf( "(thank you for using lily)" ) == 0 ){
      try{
        if( CalValue.get( "SYSTEM", "status" ).equalsIgnoreCase( "standby" ) ){
          CalValue.put( "SYSTEM", "status", "shutdown" );
          System.out.println( "go to shutdown" );
        }
      } catch( CalUnknownValueException uve ){
        // Put us in shutdown state anyway
        status.add( "System status unknown.  Shutting down anyway." );
        CalValue.put( "SYSTEM", "status", "shutdown" );
      }
      // @@@ Should signal the net is about to die and handle
      default_send( text, buffer );    // @@@ Send to all?
    } else {
      default_send( text, buffer );
    }
  }

//---------------------------------------------------------------------------

  void send_to_buffer( String name, String text ) 
  throws CalUnknownBuffException{
    Buffer tobuf = (Buffer)buffers.get( name );
    if( tobuf != null ){
      tobuf.put( text );
    } else {
      throw new CalUnknownBuffException( name );
    }
  }

//---------------------------------------------------------------------------

  void oob_send_to_buffer( String name, String text ) 
  throws CalUnknownBuffException{
    Buffer tobuf = (Buffer)buffers.get( name );
    if( tobuf != null ){
      tobuf.putOOB( text );
    } else {
      throw new CalUnknownBuffException( name );
    }
  }

//---------------------------------------------------------------------------

  public void stop(){

    System.out.println( "Stopping " + thread.getName() );
    // Tell all of our buffers to die
    Enumeration buffer_list = buffers.keys();
    while( buffer_list.hasMoreElements() ){
      String buffer_name = (String)buffer_list.nextElement();
      parse_percent( "%cal close " + buffer_name );      
    }

    // And stop the thread
    thread.stop();
    thread = null;
    System.out.println( "STOPPED" );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class LoginBox extends CalFrame{

  String message;
  Buffer tonet;

  CalButton ok;
  TabTextField loginField;
  TabTextField passwordField;
  Checkbox     doScript;

//---------------------------------------------------------------------------

  LoginBox( String m, Buffer to ){
    super( "CaL Login" );

    tonet = to;

    // Set the message to what lily prompts us
    try{
      int loc = m.lastIndexOf( "login:" );
      message = m.replace( '\r', ' ' )
                 .replace( '\n', ' ' )
                 .substring( 0, loc )
                 .trim();
    } catch( StringIndexOutOfBoundsException e ){
      message = "Welcome";
    }

    ok = new CalButton( "Ok" );
    passwordField = new TabTextField( 10, ok );
    passwordField.setEchoCharacter( '*' );
    loginField = new TabTextField( 10, passwordField );

    CalPanel login = new CalPanel();
    login.setLayout( new BorderLayout() );
    login.add( "West", new Label( "Login: " ) );
    login.add( "Center", loginField );

    CalPanel password = new CalPanel();
    password.setLayout( new BorderLayout() );
    password.add( "West", new Label( "Password: " ) );
    password.add( "Center", passwordField );
 
    doScript = new Checkbox( "Execute startup script" );
    doScript.setState( true );
    doScript.enable( false );    // @@@ This feature isn't implemented

    GridBagLayout layout = new GridBagLayout();
    setLayout( layout );
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.gridwidth = GridBagConstraints.REMAINDER;

    addObject( layout, gbc, new Label( "" ) );
    addObject( layout, gbc, new Label( message, Label.CENTER ) );
    gbc.anchor = GridBagConstraints.EAST;
    addObject( layout, gbc, login );
    addObject( layout, gbc, password );
    gbc.anchor = GridBagConstraints.WEST;
    addObject( layout, gbc, doScript );
    gbc.anchor = GridBagConstraints.CENTER;
    addObject( layout, gbc, ok );
    addObject( layout, gbc, new Label( "" ) );
    pack();
    show();
    
  }

//---------------------------------------------------------------------------

  void addObject( GridBagLayout l, GridBagConstraints c, Component o ){
    l.setConstraints( o, c );
    add( o );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){
    if( CalButton.buttonHit( evt ) ){
      // They pressed the button

      // Get the strings, and if they are blank, put in obvious garbage
      String login = loginField.getText();
      if( login.length() == 0 ){
        login = "x";
      }
      String pass  = passwordField.getText();
      if( pass.length() == 0 ){
        pass = "x";
      }
      
      // Get the data from the logins and send it
      tonet.put( login + ' ' + pass + '\n' );

      // Destroy ourselves
      dispose();
      return true;
    }

    // Other events
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class NameBox extends CalFrame{

  Buffer tonet;

  List names;

//---------------------------------------------------------------------------

  NameBox( String data, Buffer to ){
    super( "CaL Reserved Names" );
    tonet = to;

    TextArea motd = new TextArea( 10, 80 );
    motd.setEditable( false );
    CalButton ok = new CalButton( "OK" );
    names = new List( 3, false );

    StringTokenizer dataLines = new StringTokenizer( data, "\n\r", false );
    while( dataLines.hasMoreTokens() ){
      String line = dataLines.nextToken();
      if( line.indexOf( '*' ) == 0 ){
        // Part of the MOTD
        motd.appendText( line + '\n' );
      } else if( line.indexOf( "last login at" ) == 1 ){
        // Put the last login info into the MOTD
        motd.appendText( '\n' + line + '\n' );
      } else if( line.indexOf( ">>>>" ) == 1 ){
        int first_quote = 0;
        int last_quote = 0;
        // This line contains the login names
        while( first_quote != -1 ){
          System.out.println( "First quote at " + first_quote );
          first_quote = line.indexOf( '"', last_quote+1 );
          if( first_quote != -1 ){
            last_quote = line.indexOf( '"', first_quote+1 );
            try{
              String name = line.substring( first_quote+1, last_quote );
              names.addItem( name );
            } catch( StringIndexOutOfBoundsException e ){
            }
          }
          
        }
      } else {
        System.out.println( "No clue: " + line );
      }
    }

    GridBagLayout layout = new GridBagLayout();
    setLayout( layout );
    GridBagConstraints gbc = new GridBagConstraints();

    gbc.weightx = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridheight = GridBagConstraints.REMAINDER;
    addObject( layout, gbc, motd );

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.gridheight = 1;
    gbc.weightx = 0.0;
    gbc.weighty = 1.0;
    addObject( layout, gbc, names );

    gbc.fill = GridBagConstraints.NONE;
    gbc.gridheight = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.SOUTH;
    addObject( layout, gbc, ok );
    pack();
    show();

  }

//---------------------------------------------------------------------------

  void addObject( GridBagLayout l, GridBagConstraints c, Component o ){
    l.setConstraints( o, c );
    add( o );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){

    String name;

    if( CalButton.buttonHit( evt ) ){
      // They pressed the button
      
      // Get the data from the logins and send it
      name = names.getSelectedItem();
      if( name == null ){
        name = names.getItem( 0 );
      }
      tonet.put( name + '\n' );      

      // Destroy ourselves
      dispose();
      return true;
    } else if( evt.target == names ){
      if( evt.id == Event.ACTION_EVENT ){
        // Get the data from the logins and send it
        name = names.getSelectedItem();
        if( name == null ){
          name = names.getItem( 0 );
        }
        tonet.put( name + '\n' );      

        // Destroy ourselves
        dispose();
        return true;
      }
    }

    // Other events
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class BlurbBox extends CalFrame{

  Buffer tonet;

  TabTextField blurb;

//---------------------------------------------------------------------------

  BlurbBox( String data, Buffer to ){
    super( "CaL Blurb Entry" );
    tonet = to;

    TextArea motd = new TextArea( 10,80 );
    motd.setEditable( false );
    CalButton ok = new CalButton ( "OK" );
    blurb = new TabTextField( 40, ok );

    StringTokenizer dataLines = new StringTokenizer( data, "\n\r", false );
    while( dataLines.hasMoreTokens() ){
      String line = dataLines.nextToken();
      if( line.indexOf( '*' ) == 0 ){
        // Part of the MOTD
        motd.appendText( line + '\n' );
      } else if( line.indexOf( "last login at" ) == 1 ){
        // Put the last login info into the MOTD
        motd.appendText( '\n' + line + '\n' );
      } else if( line.indexOf( "blurb" ) != -1 ){
        motd.appendText( '\n' + line + '\n' );
      } else {
        System.out.println( "No clue: " + line );
      }
    }
    
    GridBagLayout layout = new GridBagLayout();
    setLayout( layout );
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.weighty = 1.0;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.ipady = 2;
    addObject( layout, gbc, new Label( "" ) );
    addObject( layout, gbc, motd );

    gbc.gridwidth = 1;
    gbc.anchor = GridBagConstraints.WEST;
    addObject( layout, gbc, new Label( "Blurb:" ) );
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addObject( layout, gbc, blurb );

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.CENTER;
    addObject( layout, gbc, ok );
    addObject( layout, gbc, new Label( "" ) );
    pack();
    show();    
  }

//---------------------------------------------------------------------------

  void addObject( GridBagLayout l, GridBagConstraints c, Component o ){
    l.setConstraints( o, c );
    add( o );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){

    String name;

    if( CalButton.buttonHit( evt ) ){
      // They pressed the button
      
      // Get the data from the logins and send it
      tonet.put( blurb.getText() + '\n' );      

      // Destroy ourselves
      dispose();
      return true;
    }

    // Other events
    return false;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class TabTextField extends TextField{

  Component next;

//---------------------------------------------------------------------------  

  TabTextField( Component n ){
    super();
    next = n;
  }

//---------------------------------------------------------------------------  

  TabTextField( int cols, Component n ){
    super( cols );
    next = n;
  }

//---------------------------------------------------------------------------  

  TabTextField( String text, Component n ){
    super( text );
    next = n;
  }

//---------------------------------------------------------------------------  

  TabTextField( String text, int cols, Component n ){
    super( text, cols );
    next = n;
  }

//---------------------------------------------------------------------------  

  public boolean handleEvent( Event evt ){

    if( evt.key == '\t' ){
      if( evt.id == Event.KEY_RELEASE ){
        next.requestFocus();
        return true;
      } else {
        // Swalllow other tab events
        return true;
      }
    }

    return false;
  }

//---------------------------------------------------------------------------  

}

//***************************************************************************

class BufferTable extends Hashtable{

//---------------------------------------------------------------------------

  public BufferTable( int initialCapacity, float loadFactor ){
    super( initialCapacity, loadFactor );
  }

//---------------------------------------------------------------------------

  public BufferTable( int initialCapacity ){
    super( initialCapacity );
  }

//---------------------------------------------------------------------------

  public BufferTable( ){
    super( );
  }

//---------------------------------------------------------------------------

  public String uniqueName( String s ){
    // Take the base name and turn it into a "standard", removing duplicates
    //  and spaces and other apparent nasties
    String ret;
    String basename;
    int co = 1;
    
    // Turn the spaces into underscores and the quotes into splats
    basename = s.replace( ' ', '_' )
                .replace( '"', '*' )
                .replace( '\'', '*' );
    ret = basename;

    while( containsKey( ret ) ){
      ret = basename + "<" + co + ">";
      co++;
    }

    return ret;    
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

