// $Id: WindowEdit.java,v 1.3 1996/11/25 14:11:28 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.net.*;
import java.util.*;

import Window;

//***************************************************************************

class CalSelectWindow extends CalWindow implements Runnable{

  Thread thread;

  CalMenu pullDowns;
  TextArea text;

  Font font;

//---------------------------------------------------------------------------

  CalSelectWindow( String n, CalStatus st,
                 Buffer fromnet, Buffer tonet, Buffer toparser ){
    super( n, st, fromnet, tonet, toparser );

    pullDowns = new CalMenu( this );

    String select = "";
    try{
      select = CalValue.get( "GLOBAL", "selected" );
    } catch( CalUnknownValueException cuve ) {
      select = "(nothing selected)";
    }

    text = new TextArea( select, 24, 80 );
    text.selectAll();
    
    setLayout( new BorderLayout() );
    setMenuBar( pullDowns );
    add( "Center", text );
    pack();
    show();

    // Start the thread that will read and process data to us
    start();
  };

//---------------------------------------------------------------------------

  public void run(){
    String line;

    running = true;
    while( running ){
      line = input.get();
      //System.out.println( line );
      if( line.indexOf( "%cal close" ) == 0 ){
        running = false;
      } else {
        // We shouldn't get anything else
      }
    }

    System.out.println( "done running" );
    dispose();

  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalInfoWindow extends CalWindow implements Runnable{

  Thread thread;

  CalMenu pullDowns;
  TextArea text;
  CalPanel buttonbox;
  CalButton bsave;
  CalButton bclear;
  CalButton breload;

  String whowhat;

  boolean editable;

//---------------------------------------------------------------------------

  CalInfoWindow( String n, CalStatus st,
                 Buffer fromnet, Buffer tonet, Buffer toparser,
                 String obj ){
    super( n, st, fromnet, tonet, toparser );
    whowhat = obj;

    pullDowns = new CalMenu( this );

    text = new TextArea( 24, 78 );
    text.setFont( new Font( "Courier", Font.PLAIN, 12 ) );

    try{
      buttonbox = new CalPanel();
      buttonbox.setLayout( new GridLayout( 0, 1, 0, 5 ) );
    } catch( IllegalArgumentException e ){ 
    }
    bsave   = new CalButton( "Save" );
    bclear  = new CalButton( "Clear" );
    breload = new CalButton( "Reload" );
    buttonbox.add( bsave );
    buttonbox.add( bclear );
    buttonbox.add( breload );
    
    setLayout( new BorderLayout() );
    setMenuBar( pullDowns );
    add( "Center", text );
    add( "East", buttonbox );
    pack();
    show();

    editable = false;

    // Start the thread that will read and process data to us
    start();
  };

//---------------------------------------------------------------------------

  public void run(){
    String line;

    // Will we even be allowed to edit this?
    if( !whowhat.equalsIgnoreCase( "me" ) ){
      // We are not editing ourselves, so we need to check moderator
      parser.putOOB( "%cal dcommand "+name+" /APPOINT ME "+whowhat+'\n' );
      
      // Assume we can't for the moment
      setEditable( false );
    } else {
      editable = true;
      setEditable( true );
    }

    // Make the request
    reload();

    running = true;
    while( running ){
      line = input.get();
      //System.out.println( line );
      if( line.indexOf( "%end" ) == 0 ){
        // End of text, so we can edit it now
        text.setEditable( editable );

      } else if( line.indexOf( "%begin" ) == 0 ){
        // The beginning of text, so clear the area and prevent editing
        text.setText( "" );
        text.setEditable( false );

      } else if( line.indexOf( "%cal close" ) == 0 ){
        running = false;
      } else if( line.indexOf( "is already a moderator of" ) != -1 ){
        editable = true;
        setEditable( true );
      } else if( line.indexOf( "no rights to appoint moderators to" ) != -1 ){
        editable = false;
        setEditable( false );
      } else {
        // Put it in the info
        try{
          text.appendText( line.substring( 2 ) );
        } catch( StringIndexOutOfBoundsException sioobe ){
          text.appendText( "\n" );
        }
      }
    }

    System.out.println( "done running" );
    dispose();

  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){

    if( CalButton.buttonHit( evt ) ){
      if( evt.target == bsave ){
        save();
        return true;
      } else if( evt.target == bclear ){
        clear();
        return true;
      } else if( evt.target == breload ){
        reload();
        return true;
      }
    }

    return super.handleEvent( evt );
  }

//---------------------------------------------------------------------------

  public void save(){
    String textstring = text.getText();
    if( !textstring.endsWith( "\n" ) ){
      // If there is no trailing newline, add one
      textstring = textstring + '\n';
    }

    int lines = 0;
    int index = 0;
    try{
      while( true ){
        if( textstring.charAt( index ) == '\n' ){
          lines++;
        }
        index++;
      }
    } catch( StringIndexOutOfBoundsException sioobe ){
    }

    // @@@ Should check to make sure we're not trying to save too
    //     many lines

    String obj;
    if( whowhat.equalsIgnoreCase( "me" ) ){
      obj = "";
    } else {
      obj = whowhat;
    }
    network.put( "#$# export_file info " + lines + ' ' + obj + '\n' );

    network.put( textstring );
  }

//---------------------------------------------------------------------------

  public void clear(){
    text.setText( "" );
  }

//---------------------------------------------------------------------------

  public void reload(){
    parser.putOOB( "%cal dcommand " + name + " /INFO " + whowhat + '\n' );
  }

//---------------------------------------------------------------------------

  public void setEditable( boolean iseditable ){
    text.setEditable( iseditable );
    bsave.enable( iseditable );
    bclear.enable( iseditable );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

