// $Id: WindowList.java,v 1.2 1996/11/25 04:25:05 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.net.*;
import java.util.*;

import Window;

//***************************************************************************

class CalListWindowThread implements Runnable{

  // @@@ Most of this thread should be reengineered so there is a 
  //     general purpose scheduler that will execute commands.

  CalListWindow parent;
  CalStatus status;
  String name;
  String command;
  Buffer parser;

  Date next_update;   // When do we next update the list

  Thread thread;

//---------------------------------------------------------------------------

  CalListWindowThread( CalListWindow p, CalStatus st, 
                       String n, String c, Buffer tn ){
    parent = p;
    status = st;
    name = n;
    command = c;
    parser = tn;
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, parent.getName() + " timer" );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  public void do_command(){
    status.add( "Updating " + name + ": " + command );
    if( !command.equals( MenuList.LIST ) ){
      String order;
      parser.putOOB( "%cal dcommand " + name + ' ' + command + '\n' );

      // Figre out when we will update the list next
      int update_time = 1;        // @@@ Should have settable time
      next_update = new Date();
      int min = next_update.getMinutes();
      next_update.setMinutes( min + update_time );
    }
  }

//---------------------------------------------------------------------------

  public void run(){
    boolean running = true;

    do_command();
    while( running ){
      try{
        thread.sleep( 1000 );
        parent.do_button_update();   // @@@  Should be in the parents thread
        if( next_update.before( new Date() ) ){
          // We are after the time for updating next
          do_command();
        }
      } catch( InterruptedException e ){
      }
    }

    System.out.println( "done running " + thread.getName() );
  }

//---------------------------------------------------------------------------

  public void stop(){
    System.out.println( "Stopping " + thread.getName() );
    if( thread != null ){
      thread.stop();
      thread = null;
    }
    System.out.println( "stopped" );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalList extends List{

//---------------------------------------------------------------------------

  CalList(){
    super();
  }

//---------------------------------------------------------------------------

  CalList( int rows, boolean multipleSelections ){
    super( rows, multipleSelections );
  }

//---------------------------------------------------------------------------

  public Dimension mimimumSize( int rows ){
    Dimension ret = super.minimumSize( rows );
    ret.width = get_width();

    return ret;
  }

//---------------------------------------------------------------------------

  public Dimension mimimumSize(){
    Dimension ret = super.minimumSize();
    ret.width = get_width();

    return ret;
  }

//---------------------------------------------------------------------------

  public Dimension preferredSize( int rows ){
    Dimension ret = super.preferredSize( rows );
    ret.width = get_width();

    return ret;
  }

//---------------------------------------------------------------------------

  public Dimension preferredSize(){
    Dimension ret = super.preferredSize();
    ret.width = get_width();

    return ret;
  }

//---------------------------------------------------------------------------

  int get_width(){
    int ret;
    ret = getFontMetrics( getFont() ).charWidth( 'W' ) * 80;
    return ret;
  }

//---------------------------------------------------------------------------

  public synchronized void clear(){
    // The List class badly implements this
    // It appears to delete every other item, and the delItem() and
    //    delItems() methods don't throw exceptions

    while( countItems() != 0 ){
      delItem( 0 );
    }
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

abstract class CalListWindow extends CalWindow implements Runnable{

  CalListWindowThread timerthread;
  Thread thread;

  String command;

  Vector strings;
  Vector buttons;

  CalMenu pullDowns;
  CalList listbox;
  CalPanel buttonbox;

  Font font;

//---------------------------------------------------------------------------

  CalListWindow( String n, CalStatus st,
                 Buffer fromnet, Buffer tonet, Buffer toparser,
                 String c ){
    super( n, st, fromnet, tonet, toparser );

    command = c;

    pullDowns = new CalMenu( this );

    listbox = new CalList( 20, false );
    font = new Font( "Courier", Font.PLAIN, 12 );
    listbox.setFont( font );
    listbox.resize( 500, 200 );

    buttons = new Vector();
    try{
      buttonbox = new CalPanel();
      buttonbox.setLayout( new GridLayout( 0, 1, 0, 5 ) );
    } catch( IllegalArgumentException e ){ 
    }

    setLayout( new BorderLayout() );
    setMenuBar( pullDowns );
    add( "Center", listbox );
    add( "East", buttonbox );
    pack();
    show();

    // Send the request to the server for our data
    // @@@ Includes the button check function
    timerthread = new CalListWindowThread( this, status, name,
                                           command, toparser );
    timerthread.start();

    // Start the thread that will read and process data to us
    start();
  };

//---------------------------------------------------------------------------

  abstract void parse_line( String line );
  // Take the line and add it to whatever list they maintain

//---------------------------------------------------------------------------

  abstract void create_strings();
  // Take the list of objects and turn it into a list of strings

//---------------------------------------------------------------------------

  void add_button( CalListButton button ){
    buttons.addElement( button );
    buttonbox.add( button );
    pack();
    show();
  }

//---------------------------------------------------------------------------

  void do_button_update(){
    CalListButton button;
    int index = listbox.getSelectedIndex();

    if( index != -1 ){
      int co = 0;
      boolean done = false;
      while( !done ){
        try{
          button = (CalListButton)buttons.elementAt( co );
          button.set_active( index );
          co++;
        } catch( ArrayIndexOutOfBoundsException e ){
          done = true;
        } // End try
      } // End while
    } // End If
  } // End method

//---------------------------------------------------------------------------

  abstract void clear_list();

//---------------------------------------------------------------------------

  void draw_list(){
    strings = new Vector();
    create_strings();
    listbox.clear();
    
    int co = 0;
    boolean done = false;
    String line;
    while( !done ){
      try{
        line = (String)strings.elementAt( co );
        listbox.addItem( CalUtil.remove( line, "\r\n" ) );
        co++;
      } catch( ArrayIndexOutOfBoundsException e ){
        done = true;
      }
    }
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){
  // @@@ was action

    if( evt.target == listbox ){
      return true;
    } else if( CalButton.buttonHit( evt ) ){
      return ((CalListButton)evt.target).pressed( evt, 
        new Integer( listbox.getSelectedIndex() ) );
    }

    return super.handleEvent( evt );
  }

//---------------------------------------------------------------------------

  public void run(){
    String line;

    running = true;
    while( running ){
      line = input.get();
      //System.out.println( line );
      if( line.indexOf( "%end" ) == 0 ){
        // End of the list, so go draw it
        draw_list();
      } else if( line.indexOf( "%begin" ) == 0 ){
        // Start of the list, so we are starting over
        clear_list();
      } else if( line.indexOf( "%cal close" ) == 0 ){
        running = false;
      } else {
        parse_line( line );
      }
    }

    System.out.println( "done running" );
    timerthread.stop();
    dispose();

  }

//---------------------------------------------------------------------------

}

//***************************************************************************

abstract class CalListButton extends CalButton{

//---------------------------------------------------------------------------

  CalListButton( String n ){
    super( n );
    enable( false );
  }

//---------------------------------------------------------------------------

  abstract boolean pressed( Event evt, Object obj );

//---------------------------------------------------------------------------

  abstract void set_active( int index );

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalCantParseException extends RuntimeException{

//---------------------------------------------------------------------------

  CalCantParseException(){
    super();
  }

//---------------------------------------------------------------------------

  CalCantParseException( String s ){
    super( s );
  }

//---------------------------------------------------------------------------

  CalCantParseException( char c ){
    super( "Character '" + c + "'" );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoInfoButton extends CalListButton{

  Vector user;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhoInfoButton( Vector u, Buffer p ){
    super( "Info" );
    user = u;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    String line;
    String nameblurb;
    String namespace;
    String name;
    int blurbindex;

    // @@@ Splitting the name and the blurb is done here because
    // I'm too much of an idiot to split it before

    try{
      line = (String)user.elementAt( index );
      nameblurb = line.substring( 2, 40 );
      blurbindex = nameblurb.indexOf( '[' );
      if( blurbindex != -1 ){
        namespace = nameblurb.substring( 0, blurbindex - 1 );
      } else {
        namespace = nameblurb;
      }
      name = namespace.trim();
      parser.putOOB( "%cal command /INFO \"" + name + "\"\n" );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String line;
    char   indicator;

    try{
      line = (String)user.elementAt( index );
      indicator = line.charAt( 1 );
      if( (indicator == '+') || (indicator == '=') ){
        active = true;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
   }

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoFingerButton extends CalListButton{

  Vector user;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhoFingerButton( Vector u, Buffer p ){
    super( "Finger" );
    user = u;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    String line;
    String nameblurb;
    String namespace;
    String name;
    int blurbindex;

    // @@@ Splitting the name and the blurb is done here because
    // I'm too much of an idiot to split it before

    try{
      line = (String)user.elementAt( index );
      nameblurb = line.substring( 2, 40 );
      blurbindex = nameblurb.indexOf( '[' );
      if( blurbindex != -1 ){
        namespace = nameblurb.substring( 0, blurbindex - 1 );
      } else {
        namespace = nameblurb;
      }
      name = namespace.trim();
      parser.putOOB( "%cal command /FINGER \"" + name + "\"\n" );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String line;
    char   indicator;

    try{
      line = (String)user.elementAt( index );
      indicator = line.charAt( 1 );
      if( (indicator == '-') || (indicator == '=') ){
        active = true;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoIgnoreButton extends CalListButton{

  Vector user;

//---------------------------------------------------------------------------

  CalWhoIgnoreButton( Vector u ){
    super( "Ignore/Unignore" );
    user = u;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String  function = "Ignore/Uningnore";
    String line;
    char   indicator;

    try{
      line = (String)user.elementAt( index );
      indicator = line.charAt( 0 );
      if( (indicator == '|') || (indicator == '>') ){
        active = true;
        function = "Unignore";
      } else if( (indicator == ' ') || (indicator == '<') ){
        active = true;
        function = "Ignore";
      } else {
        // I don't know what this character is!
        active = false;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }

    // @@@ This is difficult to implement in reality!
    active = false;

    setLabel( function );
    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoNotifyButton extends CalListButton{

  Vector user;

//---------------------------------------------------------------------------

  CalWhoNotifyButton( Vector u ){
    super( "Notifications..." );
    user = u;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoConversationButton extends CalListButton{

  Vector user;

//---------------------------------------------------------------------------

  CalWhoConversationButton( Vector u ){
    super( "Conversation..." );
    user = u;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String line;
    char   indicator;

    try{
      line = (String)user.elementAt( index );
      indicator = line.charAt( 0 );
      if( indicator == ' ' ){
        // We can only send to them if nobody is ignoring each other
        active = true;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }

    // @@@ Not a valid function yet
    active = false;

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhoWindow extends CalListWindow{

  Vector user;

  Label top;
  Label bottom;

//---------------------------------------------------------------------------

  CalWhoWindow( String n, CalStatus st,
                Buffer fromnet, Buffer tonet, Buffer toparser, 
                String c ){
    super( n, st, fromnet, tonet, toparser, "/WHO " + c );

    user = new Vector();

    add_button( new CalWhoConversationButton( user ) );
    add_button( new CalWhoFingerButton( user, toparser ) );
    add_button( new CalWhoInfoButton( user, toparser ) );
    add_button( new CalWhoNotifyButton( user ) );
    add_button( new CalWhoIgnoreButton( user ) );

    top = new Label();
    top.setFont( font );
    bottom = new Label( "", Label.CENTER );
    add( "North", top );
    add( "South", bottom );    
    pack();
    show();
  }

//---------------------------------------------------------------------------

  void parse_line( String line ){
    // Take the line and add it to whatever list they maintain

    if( line.indexOf( "Name" ) == 2 ){
      // A lame check if this is the first line
      top.setText( CalUtil.remove( line, "\r\n" ) );
    } else if( line.indexOf( "----" ) == 2 ){
      // A lame check if this is the second line
    } else if( line.indexOf( "(there are" ) == 0 ){
      // A lame check if this is the last line
      bottom.setText( CalUtil.remove( line, "\r\n" ) );
    } else {
      // Right now, we just put the string there unparsed
      user.addElement( line );

    }
  }

//---------------------------------------------------------------------------

  void clear_list(){
    user.removeAllElements();
  }

//---------------------------------------------------------------------------

  void create_strings(){
    // Take the list of objects and turn it into a list of strings

    // Since its already a list of strings - copy  @@@
    int co = 0;
    boolean done = false;
    String line;
    while( !done ){
      try{
        line = (String)user.elementAt( co );
        strings.addElement( line );
        co++;
      } catch( ArrayIndexOutOfBoundsException e ){
        done = true;
      }
    }

  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatJoinButton extends CalListButton{

  Vector disc;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhatJoinButton( Vector d, Buffer p ){
    super( "Join/Quit" );
    disc = d;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean ismember( int index ) throws CalCantParseException{
    // Are we a member?
    boolean ret = false;

    String line;
    char   indicator;

    try{
      line = (String)disc.elementAt( index );
      indicator = line.charAt( 0 );
      if( indicator == '*' ){
        ret = true;
      } else if( indicator == ' ' ){
        ret = false;
      } else {
        // What is this?
        throw new CalCantParseException( indicator );
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
      throw new CalCantParseException( "Array OOB" );
    } catch( StringIndexOutOfBoundsException se ){
      throw new CalCantParseException( "String OOB" );
    }

    return ret;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    boolean member;
    String line;
    String name;
    String cmd;

    try{
      member = ismember( index );
      if( member ){
        cmd = "/QUIT";
      } else {
        cmd = "/JOIN";
      }

      line = (String)disc.elementAt( index );
      name = line.substring( 2, 12 );
      parser.putOOB( "%cal command " + cmd + ' ' + name + '\n' );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    } catch( CalCantParseException ccpe ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String function = "Join/Quit";
    boolean member;

    try{
      member = ismember( index );
      if( member ){
        function = "Quit";
        active = true;
      } else {
        function = "Join";
        active = true;
      }
    } catch( CalCantParseException ccpe ){
    }

    setLabel( function );
    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatInfoButton extends CalListButton{

  Vector disc;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhatInfoButton( Vector d, Buffer p ){
    super( "Info" );
    disc = d;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    boolean member;
    String line;
    String name;

    try{
      line = (String)disc.elementAt( index );
      name = line.substring( 2, 12 );
      //parser.putOOB( "%cal command /INFO " + name + '\n' );
      parser.putOOB( "%cal infoedit " + name + '\n' );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String line;
    char   indicator;

    try{
      line = (String)disc.elementAt( index );
      indicator = line.charAt( 1 );
      if( indicator == '+' ){
        active = true;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
   }

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatReviewButton extends CalListButton{

  Vector disc;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhatReviewButton( Vector d, Buffer p ){
    super( "Review" );
    disc = d;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    boolean member;
    String line;
    String name;

    try{
      line = (String)disc.elementAt( index );
      name = line.substring( 2, 12 );
      parser.putOOB( "%cal command /REVIEW " + name + '\n' );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;

  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;
    String line;
    char   indicator;

    try{
      line = (String)disc.elementAt( index );
      indicator = line.charAt( 0 );
      if( indicator == '*' ){
        active = true;
      }
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatFingerButton extends CalListButton{

  // This is actually the /WHAT command on a particular discussion
  // but it is so much like /FINGER that... well...

  Vector disc;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhatFingerButton( Vector d, Buffer p ){
    super( "Finger" );
    disc = d;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    boolean member;
    String line;
    String name;

    try{
      line = (String)disc.elementAt( index );
      name = line.substring( 2, 12 );
      parser.putOOB( "%cal command /WHAT " + name + '\n' );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = true;
    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatConversationButton extends CalListButton{

  Vector disc;

//---------------------------------------------------------------------------

  CalWhatConversationButton( Vector d ){
    super( "Conversation..." );
    disc = d;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;

    // @@@ Not installed yet

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatMembersButton extends CalListButton{

  Vector disc;
  Buffer parser;

//---------------------------------------------------------------------------

  CalWhatMembersButton( Vector d, Buffer p ){
    super( "Members..." );
    disc = d;
    parser = p;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    int index = ((Integer)obj).intValue();
    String line;
    String name;

    try{
      line = (String)disc.elementAt( index );
      name = line.substring( 2, 12 );
      parser.putOOB( "%cal wholist " + name + '\n' );
    } catch( ArrayIndexOutOfBoundsException ae ){
    } catch( StringIndexOutOfBoundsException se ){
    }
    
    return true;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = true;
    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatModerateButton extends CalListButton{

  Vector disc;

//---------------------------------------------------------------------------

  CalWhatModerateButton( Vector d ){
    super( "Moderate..." );
    disc = d;
  }

//---------------------------------------------------------------------------

  boolean pressed( Event evt, Object obj ){
    return false;
  }

//---------------------------------------------------------------------------

  void set_active( int index ){
    boolean active = false;

    // @@@ Not installed yet

    enable( active );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalWhatWindow extends CalListWindow{

  Vector disc;

  Label top;
  Label bottom;

//---------------------------------------------------------------------------

  CalWhatWindow( String n, CalStatus st,
                 Buffer fromnet, Buffer tonet, Buffer toparser, 
                 String c ){
    super( n, st, fromnet, tonet, toparser, "/WHAT " + c );

    disc = new Vector();

    add_button( new CalWhatConversationButton( disc ) );
    add_button( new CalWhatFingerButton( disc, toparser ) );
    add_button( new CalWhatInfoButton( disc, toparser ) );
    add_button( new CalWhatReviewButton( disc, toparser ) );
    add_button( new CalWhatJoinButton( disc, toparser ) );
    add_button( new CalWhatMembersButton( disc, toparser ) );
    add_button( new CalWhatModerateButton( disc ) );

    top = new Label();
    top.setFont( font );
    bottom = new Label( "", Label.CENTER );
    add( "North", top );
    add( "South", bottom );    
    pack();
    show();
  }

//---------------------------------------------------------------------------

  void parse_line( String line ){
    // Take the line and add it to whatever list they maintain

    if( line.indexOf( "Name" ) == 2 ){
      // A lame check if this is the first line
      top.setText( CalUtil.remove( line, "\r\n" ) );
    } else if( line.indexOf( "----" ) == 2 ){
      // A lame check if this is the second line
    } else if( line.indexOf( "(there are" ) == 0 ){
      // A lame check if this is the last line
      bottom.setText( CalUtil.remove( line, "\r\n" ) );
    } else {
      // Right now, we just put the string there unparsed
      disc.addElement( line );

    }
  }

//---------------------------------------------------------------------------

  void clear_list(){
    disc.removeAllElements();
  }

//---------------------------------------------------------------------------

  void create_strings(){
    // Take the list of objects and turn it into a list of strings

    // Since its already a list of strings - copy  @@@
    int co = 0;
    boolean done = false;
    String line;
    while( !done ){
      try{
        line = (String)disc.elementAt( co );
        strings.addElement( line );
        co++;
      } catch( ArrayIndexOutOfBoundsException e ){
        done = true;
      }
    }

  }

//---------------------------------------------------------------------------

}

//***************************************************************************

