// $Id: WindowText.java,v 1.5 1996/11/25 04:26:13 asf Exp $

//    This file is part of Cafe au Lys
//    Copyright (C) 1996  Allen S. Firstenberg
// 
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
// 
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
// 
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import java.awt.*;
import java.net.*;
import java.util.*;

import Window;

// @@@ There are MANY dependencies in here on an 80 column window
//     with a fixed width font.

//***************************************************************************

class LoCalTextWindow extends CalPanel implements Runnable{

  LineCounter linecounter;

  CalArea        area;
  CalField       field;
  CalButton      pause;

  CalStatus status;
  Buffer toparser;
  Buffer tonetwork;
  Buffer fromnet;
  String name;

  Thread thread;
  boolean running;

//---------------------------------------------------------------------------

  LoCalTextWindow( String n, CalStatus st, Buffer fn, Buffer tn, Buffer tp ){
    super();

    fromnet = fn;
    toparser = tp;
    tonetwork = tn;
    status = st;
    name = n;

    linecounter = new LineCounter( n, 24 );

    area  = new CalArea( linecounter );
    pause = new CalButton( "Pause" );
    try{
      pause.setFont( new Font( "Courier", Font.PLAIN, 
                               CalValue.getInt( "GLOBAL", "fontsize" ) ) );
    } catch( CalUnknownValueException cuve ){
      System.out.println( "FAILED to set font for button" );
      pause.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    }

    try{
      if( CalValue.get("SYSTEM", "platform").equalsIgnoreCase("Netscape") &&
          CalValue.get("SYSTEM", "os").equalsIgnoreCase("UNIX")){
        System.out.println( "Netscape" );
        field = new CalNetscapeField( tonetwork, fromnet, linecounter, area );
      } else {
        System.out.println( "not Netscape" );
        field = new CalWorkingField( tonetwork, fromnet, linecounter, area );
      }
    } catch( CalUnknownValueException cuve ){
      System.out.println( "Couldn't determine browser" );
      field = new CalWorkingField( tonetwork, fromnet, linecounter, area );
    }

    // Do layout
    GridBagLayout layout = new GridBagLayout();
    setLayout( layout );
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.ipady = 10;

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    try{
      addObject( layout, gbc, 
                 new CalSpace(CalValue.getInt("GLOBAL", "spacing")) );
    } catch( CalUnknownValueException cuve1 ){
    }

    gbc.fill = GridBagConstraints.HORIZONTAL;
    addObject( layout, gbc, area );
    gbc.fill = GridBagConstraints.NONE;

    gbc.gridheight = GridBagConstraints.RELATIVE;
    gbc.gridwidth = 1;
    addObject( layout, gbc, field );

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addObject( layout, gbc, pause );

    gbc.gridheight = GridBagConstraints.REMAINDER;
    try{
      addObject( layout, gbc, 
                 new CalSpace(CalValue.getInt("GLOBAL", "spacing")) );
    } catch( CalUnknownValueException cuve1 ){
    }

    start();
  }

//---------------------------------------------------------------------------

  void addObject( GridBagLayout l, GridBagConstraints c, Object o ){
    l.setConstraints( (Component)o, c );
    add( (Component)o );
  }

//---------------------------------------------------------------------------

  public void start(){
    if( thread == null ){
      thread = new Thread( this, name );
      thread.start();
    }
  }

//---------------------------------------------------------------------------

  public void stop(){
    System.out.println( "Stopping " + thread.getName() );
    if( thread != null ){
      thread.stop();
      thread = null;
    }
    System.out.println( "stopped" );
  }

//---------------------------------------------------------------------------

  public void run(){
    running = true;

    while( running ){
      int size = fromnet.size();

      while( (size > 0) && (running) ){
        if( !linecounter.isMaxed() || fromnet.isOOB() ){
          // We can get data either if there is OOB data existing, or
          // we haven't run out of room to print stuff

          // Get the string from the buffer, blocking
          String newstring = fromnet.get();
          if( newstring.indexOf( '%' ) == 0 ){
            percent_command( newstring );
          } else {
            String leftover = area.storeText( newstring );
            if( leftover != null ){
              fromnet.push( leftover );
            }
          }

        } else {
          try{
            thread.sleep( 1000 );
          } catch( InterruptedException e ){
          }
        }
        size = fromnet.size();
        pause.setLabel( String.valueOf( size ) );
        pause.enable();
      } 
      // There is either nothing in the buffer, or we're closing up

      pause.setLabel( "Pause" );
      pause.disable();

      if( running ){
        // If we are still going, wait a second so we yield time
        try{
          thread.sleep( 1000 );
        } catch( InterruptedException e ){
        }
      }

    } // End while running 
    System.out.println( "not running" );
  }

//---------------------------------------------------------------------------

  void percent_command( String line ){

    String parameter[] = CalUtil.split( line );
    String command = parameter[0];

    if( command.equalsIgnoreCase( "%cal" ) ){
      if( parameter[1].equalsIgnoreCase( "close" ) ){
        System.out.println( "got close" );
        running = false;
        // buff.stop();
      } else if( parameter[1].equalsIgnoreCase( "redirect" ) ){
        // Don't do anything
      } else if( parameter[1].equalsIgnoreCase( "nodirect" ) ){
        // Don't do anything
      } else {
        fromnet.push( "Unknown command: '" + line + "'\n" );
      }
    } else if( command.equalsIgnoreCase( "%g" ) ){
      // Send a signal 
      System.err.print( '\007' );     // @@@ Need real audio
      System.err.flush();
    } else {
      fromnet.push( "Unknown command: '" + line + "'\n" );
    }

  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){
  // @@@ was action

    if( evt.id == Event.WINDOW_DESTROY ){
      return true;
//    } else if( evt.target instanceof MenuItemAction ){
//      return ((MenuItemAction)evt.target).action( evt, new Object() );
    } else if( evt.id == Event.GOT_FOCUS ){
      System.out.println( "CalWindow got focus" );
      return super.handleEvent( evt );
    } else if( evt.id == Event.KEY_ACTION_RELEASE ){
      // @@@ None of this works either, either here or in handleEvent
      // Function keys?
      if( evt.key == Event.F11 ){
        // @@@ more dorking
        //field.requestFocus();
        return true;
      } else if( evt.key == Event.F12 ){
        // @@@ done because the above is dorked
        // Trigger the MORE button
        linecounter.clear();
      
        // Get out
        return true;
      }
    } else if( evt.id == Event.KEY_PRESS ){
      // @@@ None of this works either, either here or in handleEvent
      // Transfer focus to the entry area.
      // @@@ This translate doesn't quite do what you think
      // field.requestFocus();
      // evt.translate( field.location().x, field.location().y );
      return false;
    } else if( (evt.target == pause) && (CalButton.buttonHit( evt )) ){
      System.out.println( "Pause button" );
      // Resete the line counter
      linecounter.clear();
      return true;
    }

    return super.handleEvent( evt );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalTextWindow extends CalWindow{

  LineCounter linecounter;

  CalArea        area;
  CalField       field;
  CalButton      pause;
  CalMenu        pullDowns;

  Buffer fromnet;
  Buffer tonet;
  Buffer parser;

//---------------------------------------------------------------------------

  CalTextWindow( String n, CalStatus st,
                 Buffer fn, Buffer tn, Buffer tp ){
    super( n, st, fn, tn, tp );

    fromnet = fn;
    tonet = tn;
    parser = tp;

    linecounter = new LineCounter( n, 24 );

    area  = new CalArea( linecounter );
    pause = new CalButton( "Pause" );
    pullDowns = new CalMenu( this );

    try{
      if( CalValue.get("SYSTEM", "platform").equalsIgnoreCase("Netscape") &&
          CalValue.get("SYSTEM", "os").equalsIgnoreCase("UNIX")){
        System.out.println( "Netscape" );
        field = new CalNetscapeField( tonet, fromnet, linecounter, area );
      } else {
        System.out.println( "not Netscape" );
        field = new CalWorkingField( tonet, fromnet, linecounter, area );
      }
    } catch( CalUnknownValueException cuve ){
      System.out.println( "Couldn't determine browser" );
      field = new CalWorkingField( tonet, fromnet, linecounter, area );
    }

    // Do layout
    GridBagLayout layout = new GridBagLayout();
    setLayout( layout );
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.ipady = 10;

    setMenuBar( pullDowns );

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    try{
      addObject( layout, gbc, 
                 new CalSpace(CalValue.getInt("GLOBAL", "spacing")) );
    } catch( CalUnknownValueException cuve1 ){
    }

    gbc.fill = GridBagConstraints.HORIZONTAL;
    addObject( layout, gbc, area );
    gbc.fill = GridBagConstraints.NONE;

    gbc.gridheight = GridBagConstraints.RELATIVE;
    gbc.gridwidth = 1;
    addObject( layout, gbc, field );

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    addObject( layout, gbc, pause );

    gbc.gridheight = GridBagConstraints.REMAINDER;
    try{
      addObject( layout, gbc, 
                 new CalSpace(CalValue.getInt("GLOBAL", "spacing")) );
    } catch( CalUnknownValueException cuve1 ){
    }

    pack();
    show();

//    // Start the threads
//    pause.start();
    start();

//    // Block on terminating until the thread stops
//    try{
//      pause.join();      // Stop the thread
//    } catch( InterruptedException e ){
//      System.out.println( "Interrupted Exception" );
//    }
//    System.out.println( "Dispose 2" );
//    super.dispose();
//    System.out.println( "Dispose 3" );

  }

//---------------------------------------------------------------------------

  void addObject( GridBagLayout l, GridBagConstraints c, Object o ){
    l.setConstraints( (Component)o, c );
    add( (Component)o );
  }

//---------------------------------------------------------------------------

  public void run(){
    running = true;

    while( running ){
      int size = fromnet.size();

      while( (size > 0) && (running) ){
        if( !linecounter.isMaxed() || fromnet.isOOB() ){
          // We can get data either if there is OOB data existing, or
          // we haven't run out of room to print stuff

          // Get the string from the buffer, blocking
          String newstring = fromnet.get();
          if( newstring.indexOf( '%' ) == 0 ){
            percent_command( newstring );
          } else {
            String leftover = area.storeText( newstring );
            if( leftover != null ){
              fromnet.push( leftover );
            }
          }

        } else {
          try{
            thread.sleep( 1000 );
          } catch( InterruptedException e ){
          }
        }
        size = fromnet.size();
        pause.setLabel( String.valueOf( size ) );
        pause.enable();
      } 
      // There is either nothing in the buffer, or we're closing up

      pause.setLabel( "Pause" );
      pause.disable();

      if( running ){
        // If we are still going, wait a second so we yield time
        try{
          thread.sleep( 1000 );
        } catch( InterruptedException e ){
        }
      }

    } // End while running 
    System.out.println( "not running" );
    dispose();
  }

//---------------------------------------------------------------------------

  void percent_command( String line ){

    String parameter[] = CalUtil.split( line );
    String command = parameter[0];

    if( command.equalsIgnoreCase( "%cal" ) ){
      if( parameter[1].equalsIgnoreCase( "close" ) ){
        System.out.println( "got close" );
        running = false;
        // buff.stop();
      } else if( parameter[1].equalsIgnoreCase( "redirect" ) ){
        // Don't do anything
      } else if( parameter[1].equalsIgnoreCase( "nodirect" ) ){
        // Don't do anything
      } else {
        fromnet.push( "Unknown command: '" + line + "'\n" );
      }
    } else if( command.equalsIgnoreCase( "%g" ) ){
      // Send a signal 
      System.err.print( '\007' );     // @@@ Need real audio
      System.err.flush();
    } else {
      fromnet.push( "Unknown command: '" + line + "'\n" );
    }

  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event evt ){
  // @@@ was action

    if( evt.id == Event.KEY_ACTION_RELEASE ){
      // @@@ None of this works either, either here or in handleEvent
      // Function keys?
      if( evt.key == Event.F11 ){
        // @@@ more dorking
        //field.requestFocus();
        return true;
      } else if( evt.key == Event.F12 ){
        // @@@ done because the above is dorked
        // Trigger the MORE button
        linecounter.clear();
      
        // Get out
        return true;
      }
    } else if( evt.id == Event.KEY_PRESS ){
      // @@@ None of this works either, either here or in handleEvent
      // Transfer focus to the entry area.
      // @@@ This translate doesn't quite do what you think
      // field.requestFocus();
      // evt.translate( field.location().x, field.location().y );
      return false;
    } else if( (evt.target == pause) && (CalButton.buttonHit( evt )) ){
      // Resete the line counter
      linecounter.clear();
      return true;
    }

    return super.handleEvent( evt );
  }

//---------------------------------------------------------------------------

  CalArea getArea(){
    return area;
  }

//---------------------------------------------------------------------------

  CalField getField(){
    return field;
  }

//---------------------------------------------------------------------------

  LineCounter getLines(){
    return linecounter;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalArea extends CalPanel{

  LineCounter lines;

  Vector text;        // The text entered

  GrCalAreaCanvas canvas;
  GrCalScrollbar right;

  int num = 24;

//---------------------------------------------------------------------------

  CalArea( LineCounter l ){
    super();
    lines = l;

    text = new Vector();  // Create a place to store all our text

    // Create the work areas and draw
    canvas = new GrCalAreaCanvas( text );
    right = new GrCalScrollbar( Scrollbar.VERTICAL, 
                                0, num, 
                                0, 0 );
    right.setPageIncrement( num );
    setLayout( new BorderLayout() );
    add( "Center", canvas );
    add( "East", right );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent(Event evt) {
    if (evt.target instanceof Scrollbar) {
      int val = ((Scrollbar)evt.target).getValue();
      canvas.setFirstLine( val );
      repaint();
      return true;
    } 
    return super.handleEvent( evt );;
  }


//---------------------------------------------------------------------------

  public String storeText( String s ){
    // Attempt to print the text, return any portion that could not
    // be printed or null if all printed

    String ret = null;

    int maxLineWidth = 80;    // @@@ Automatically compute

    // Ensure resize correctly
    charLines();

    // Get rid of unsightly characters  (this is cheating)
    //String ns = s.replace( '\n', ' ' ).replace( '\r', ' ' );
    String ns = CalUtil.remove( s, "\n\r" );

    // Add it to the list of lines, one line at a time
    int pos = 0;
    boolean done = false;
    boolean leftover = true;
    while( !lines.isMaxed() && !done ){
      try{
        text.addElement( ns.substring( pos, pos+maxLineWidth ) );
      } catch( StringIndexOutOfBoundsException e ){
        // There was more than maxLineWidth characters - we're at the end
        try{
          text.addElement( ns.substring( pos ) );
        } catch( StringIndexOutOfBoundsException ei ){
        }
        done = true;
        leftover = false;
      }
      lines.inc();
      pos += maxLineWidth;
    }

    // If there was stuff not printed, send it back
    if( leftover ){
      ret = ns.substring( pos );
    }

    // Set the scrollbar
    int size = text.size();
    right.setMaximum( size-num );    // The last value is the top of
                                     // the screen
    right.setValue( size );

    // Put it on the canvas
    canvas.setLastLine( size );

    return ret;
  }

//---------------------------------------------------------------------------

  public void charLines(){

    // Load the data from the text portion
    num = canvas.numLines();

    // Set some values in the scrollbar
    right.setVisible( num );
    right.setPageIncrement( num );
    
    // Then save it for the overflow line counter
    lines.setMax( num );
  }

//---------------------------------------------------------------------------

  public void pgup(){
    right.pgup();
    canvas.setFirstLine( right.getValue() );
  }

//---------------------------------------------------------------------------

  public void pgdn(){
    right.pgdn();
    canvas.setFirstLine( right.getValue() );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class GrCalAreaCanvas extends Canvas{

  Vector text;
  int firstline;

  Font font;

  boolean selected;
  Coordinate selStart;
  Coordinate selEnd;
  Coordinate selMark;
  
//---------------------------------------------------------------------------

  GrCalAreaCanvas( Vector t ){
    super();
    text = t;
    firstline = 0;

    // This won't work since you don't have a toolkit here???
    //String fonts[] = getToolkit().getFontList();
    //int cof;
    //for( cof=0; cof<fonts.length; cof++ ){
    //  System.out.println( fonts[cof] );
    //}

    try{
      font = new Font( "Courier", Font.PLAIN, 
                       CalValue.getInt( "GLOBAL", "fontsize" ) );
    } catch( CalUnknownValueException cuve ){
      font = new Font( "Courier", Font.PLAIN, 12 );
    }
    setFont( font );

    selected = false;
    selMark = new Coordinate( 0, 0 );
    selStart = selMark;
    selEnd = selMark;
  }

//---------------------------------------------------------------------------

  public void paint( Graphics g ){

    int numLines = numLines();
    FontMetrics fm = getFontMetrics( getFont() );
    int fontHeight   = fm.getHeight();
    int y = fontHeight;
    int co;
    String string;
    String selString = "";

    for( co=firstline; co<firstline+numLines; co++ ){

      try{
        string = (String)text.elementAt( co );
        g.drawString( string, 0, y );

        // Do some processing if text has been selected
        if( selected ){

          // Compute what line we are on, the coordiantes of the
          // first and last characters on this line, and some sizes.
          // Used for comparison later
          int endline = fm.stringWidth( string );
          int fontw = getFontMetrics( font ).charWidth( 'W' );
          int line = co - firstline;
          Coordinate firstchar = new Coordinate( 0, line );
          Coordinate lastchar = new Coordinate( 79, line );

          if( firstchar.le( selEnd ) && lastchar.ge( selStart ) ){
            // Some part of this line is in the selected area

            // Figure out what pixel to actually start on and
            // how far to go
            int start = 0;
            if( selStart.y() == line ){
              // We may need to move the starting position
              start = fontw * selStart.x();
            }
            if( selEnd.y() == line ){
              // It doesn't go all the way across, so change the end
              endline = fontw * selEnd.x();
            }
            int width = endline - start;

            // Draw the box around the text
            g.setXORMode( Color.lightGray);
            g.fillRect( start, y-fontHeight+3, width, fontHeight );
            g.setPaintMode();

            // Compute the portions of the string to add
            int bi = col( start );
            if( bi < 0 ) bi = 0;
            int ei = bi + col( width );
            try{
              selString = selString + string.substring( bi, ei ) + "\n";
            } catch( StringIndexOutOfBoundsException sioobe1 ){
              // Add the rest of the string
              try{
                selString = selString + string.substring( bi ) + "\n";
              } catch( StringIndexOutOfBoundsException sioobe2 ){
                // Forget it.  Don't add anything
              }            
            }
          }
        }

        y += fontHeight;
      } catch( ArrayIndexOutOfBoundsException e ){
      }      

    }

    if( selected ){
      // @@@
      System.out.println( "Selected: '" + selString + "'" );
      CalValue.put( "GLOBAL", "selected", selString );
    }

  }

//---------------------------------------------------------------------------

  public void setFirstLine( int l ){
    firstline = l;
    repaint();
  }

//---------------------------------------------------------------------------

  public void setLastLine( int l ){
    int num = numLines();
    if( l < num ){
      setFirstLine( 0 );
    } else {
      setFirstLine( l - num );
    }
  }

//---------------------------------------------------------------------------

  public int numLines(){
    int windowHeight = size().height;
    int fontHeight   = getFontMetrics( getFont() ).getHeight();
    int num = (windowHeight / fontHeight);
    return num;
  }

//---------------------------------------------------------------------------

  public Dimension minimumSize() {
    int fontw = getFontMetrics( font ).charWidth( 'W' );
    int fonth = getFontMetrics( font ).getHeight();
    return new Dimension(80*fontw, 24*fonth);
  }

//---------------------------------------------------------------------------

  public Dimension preferredSize() {
    return minimumSize();
  }

//---------------------------------------------------------------------------

  int row( int y ){
    // Given a y coordinate, what row is it in relative to the screen
    // 0 is the first row
    int fonth = getFontMetrics( font ).getHeight();
    int ret = y / fonth;
    return ret;
  }

//---------------------------------------------------------------------------

  int col( int x ){
    // Given an x coordiante, what column is it in relative to the
    // screen.  0 is the first printed column.
    // @@@ this assumes fixed width fonts
    int fontw = getFontMetrics( font ).charWidth( 'W' );
    int ret = x / fontw;
    return ret;
  }

//---------------------------------------------------------------------------

  public boolean mouseDown( Event evt, int x, int y ){

    if( selected ){
      selected = false;
      repaint();
    }

    selMark = new Coordinate( col(x), row(y) );
    selStart = selMark;
    selEnd = selMark;
    
    return true;
  }

//---------------------------------------------------------------------------

  public boolean mouseDrag( Event evt, int x, int y ){
    selected = true;

    int r = row( y );
    int c = col( x );

    Coordinate oldStart = selStart;
    Coordinate oldEnd = selEnd;

    Coordinate here = new Coordinate( c, r );
    if( here.lt( selMark ) ){
      selStart = here;
      selEnd = selMark;
    } else if( here.gt( selMark ) ){
      selStart = selMark;
      selEnd = here;
    } else {
      selected = false;
    }

    // Only repaint if there is a change
    if( oldStart.ne( selStart ) || oldEnd.ne( selEnd ) ){
      repaint();
    }

    return true;
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class GrCalScrollbar extends Scrollbar{

//---------------------------------------------------------------------------

  GrCalScrollbar( int orientation,
                  int value, int visible,
                  int minimum, int maximum ){
    super( orientation, value, visible, minimum, maximum );
  }

//---------------------------------------------------------------------------

  public void setVisible( int visible ){
    setValues( getValue(), visible, getMinimum(), getMaximum() );
  }

//---------------------------------------------------------------------------

  public void setMinimum( int min ){
    setValues( getValue(), getVisible(), min, getMaximum() );
  }

//---------------------------------------------------------------------------

  public void setMaximum( int max ){
    setValues( getValue(), getVisible(), getMinimum(), max );
  }

//---------------------------------------------------------------------------

  public void pgup(){
    int value = getValue();
    int inc   = getPageIncrement();
    setValue( value - inc );
  }

//---------------------------------------------------------------------------

  public void pgdn(){
    int value = getValue();
    int inc   = getPageIncrement();
    setValue( value + inc );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

interface CalField{
  // @@@ ?  No interface in an interface
}

//***************************************************************************

class CalWorkingField extends TextArea implements CalField{

  Buffer tonet;
  Buffer fromnet;

  Socket network;

  LineCounter lines;

  CalArea area;

  Vector sent_to;
  int sent_to_ptr;

  Vector sent_msg;
  int sent_msg_ptr;
  String sent_msg_save;

//---------------------------------------------------------------------------

  CalWorkingField( Buffer b, Buffer b2, LineCounter l, CalArea a ){

    //super( 2, 80 );
    super( 1, 80 );

    try{
      setFont( new Font( "Courier", Font.PLAIN, 
                         CalValue.getInt( "GLOBAL", "fontsize" ) ) );
    } catch( CalUnknownValueException cuve ){
      setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    }

    tonet = b;
    fromnet = b2;

    lines = l;

    area = a;

    sent_to = new Vector();
    sent_to_ptr = -1;

    sent_msg = new Vector();
    sent_msg_ptr = -1;
    sent_msg_save = "";

  }

//---------------------------------------------------------------------------

  boolean insertName( String value ){
    // Only do something if nothing is entered
    //    @@@ should be if its the first
    int l = getText().length();
    if( l == 0 ){
      try{
        // Insert the name and the semicolon at the beginning

        // @@@ It is arguable this shouldn't always be a semicolon,
        // because of how we are doing "sendee" parsing at a newline.

        String name = CalValue.get( "GLOBAL", value );
        int len = name.length();

        insertText( name + ';', 0 );

        return true;
      } catch( CalUnknownValueException e ){
        // Don't do any processing.  No value.
        return false;
      }
    } else {
      // Do not do any processing on it, there is already data
      return false;
    }
  }

//---------------------------------------------------------------------------

  int findEndName(){
    // Try to treat the string as a name and a message, seperated by
    // a semicolon (@@@ use recip_regexp!).  The name cannot have
    // spaces.
    //
    // Return -1 if there is no name, otherwise return the index
    // of the ;
    
    int ret = -1;
    String text = getText();
    int firstspace = text.indexOf( ' ' );
    int firstsemi = text.indexOf( ';' );
    if( (firstsemi != -1) &&
        ( (firstsemi < firstspace) || (firstspace == -1) ) ){
        ret = firstsemi;
    }

    return ret;
  }
  
//---------------------------------------------------------------------------
  
  void replaceName( String name ){
    // Search for a name, if any, and replace it.  If there is no
    // name, just insert one.
    // Name should have trailing ;

    int semicolon = findEndName();
    if( semicolon != -1 ){
      // There was a name
      replaceText( name, 0, semicolon+1 );
    } else {
      // No name already there.  Insert.
      insertText( name, 0 );
    }
  }

//---------------------------------------------------------------------------

  void scrollMsg( int dir ){

    System.out.println( "scrollMsg " + dir );

    String newtext = "**error**";
  
    if( sent_msg_ptr == -1 ){
      sent_msg_save = getText();
    }

    sent_msg_ptr += dir;

    if( sent_msg_ptr == -2 ){
      // Looped bakcwards
      sent_msg_ptr = sent_msg.size() - 1;
    }

    try{
      newtext = (String)sent_msg.elementAt( sent_msg_ptr );
    } catch( ArrayIndexOutOfBoundsException except ){
      newtext = sent_msg_save;
      sent_msg_ptr = -1;
    }

    setText( newtext );
  }

//---------------------------------------------------------------------------

  void handleEnter(){

    // Get the value in the entry area
    String text = getText();

    // Remove any newlines that might be hiding in here
    text = CalUtil.remove( text, "\r\n" );

    // Clear the line counter if we're not maxed out
    // Do this before we add the text to the buffer so the message
    //   itself can't max out the buffer
    if( !lines.isMaxed() ){
      lines.clear();
    }

    // Send it to the network and the return buffer
    tonet.put( text + '\n' );
    fromnet.put( text + '\n' );

    // Try to determine who we just sent it to
    // @@@ Should be using recip-regexp here!
    int firstsemi = findEndName();
    if( firstsemi != -1 ){
      String last_send = text.substring( 0, firstsemi );

      // Set this as our last send
      CalValue.put( "GLOBAL", "last_send", last_send );
       // Put it in the list of people we've sent to
      sent_to.insertElementAt( last_send, 0 );
    }

    // Store the message
    sent_msg.insertElementAt( text, 0 );

    // We might have been using the sent lists, so reset them to the beginning
    sent_to_ptr = -1;
    sent_msg_ptr = -1;

    // Clear the entry
    setText( "" );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event e ){
    //System.out.println( "Mod: " + e.modifiers );
    //System.out.println( "key: " + e.key + " F12: " + Event.F12 );
    //System.out.println( "id: " +e.id+ " KAR: " + Event.KEY_ACTION_RELEASE );

    if( e.id == Event.KEY_PRESS ){
      if( e.key == ';' ){
        return insertName( "last_send" );
      } else if( e.key == ':' ){
        return insertName( "last_recip" );
      } else if( e.key == '=' ){
        return insertName( "last_group" );
      } else if( e.key == '\t' ){
        if( e.shiftDown() ){
          // Shifted.  Go backwards
          sent_to_ptr--;
          if( sent_to_ptr == -2 ){
            // Looped backwards
            sent_to_ptr = sent_to.size() - 1;
          }
        } else {
          // No shift.  Advance
          sent_to_ptr++;
        }
        try{
          String name = (String)sent_to.elementAt( sent_to_ptr );
          replaceName( name + ';' );
        } catch( ArrayIndexOutOfBoundsException except ){
          replaceName( "" );
          sent_to_ptr = -1;
        }
        return true;
      }

    } else if( e.id == Event.KEY_RELEASE ){
      // Normal keys?
      if( e.key == '\n' ){
        handleEnter();
        return true;
      } else if( e.metaDown() && (e.key == 'm') ){
        // @@@ This is dorked too
        // Trigger the MORE button
        lines.clear();
      
        // Get out
        return true;
      }
    } else if( e.id == Event.KEY_ACTION_RELEASE ){
      // Function keys?
      if( e.key == Event.F12 ){
        // @@@ done because the above is dorked
        // Trigger the MORE button
        lines.clear();
      
        // Get out
        return true;
      } else if( e.key == Event.UP ){
        // Move to the last message in the list
        scrollMsg( 1 );
      } else if( e.key == Event.DOWN ){
        // Move to the next message in the list
        scrollMsg( -1 );
      } else if( e.key == Event.PGUP ){
        // Move the scroll bar up
        area.pgup();
      } else if( e.key == Event.PGDN ){
        // Move the scroll bar down
        area.pgdn();
      }

      // Cannot get it to recognize "normal" keys in this section,
      // they are swallowed up by the text entry area.

    }
    return super.handleEvent( e );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class CalNetscapeField extends TextField implements CalField{

  Buffer tonet;
  Buffer fromnet;

  Socket network;

  LineCounter lines;

  CalArea area;

  Vector sent_to;
  int sent_to_ptr;

  Vector sent_msg;
  int sent_msg_ptr;
  String sent_msg_save;

//---------------------------------------------------------------------------

  CalNetscapeField( Buffer b, Buffer b2, LineCounter l, CalArea a ){

    super( 80 );
    try{
      setFont( new Font( "Courier", Font.PLAIN, 
                         CalValue.getInt( "GLOBAL", "fontsize" ) ) );
    } catch( CalUnknownValueException cuve ){
      setFont( new Font( "Courier", Font.PLAIN, 12 ) );
    }

    tonet = b;
    fromnet = b2;

    lines = l;
    area = a;

    sent_to = new Vector();
    sent_to_ptr = -1;

    sent_msg = new Vector();
    sent_msg_ptr = -1;
    sent_msg_save = "";

  }

//---------------------------------------------------------------------------

  boolean insertName( String value ){
    // Insert the name stored in the GLOBAL value table for this value
    // Only do something if nothing is entered
    //    @@@ should be if its the first
    //    @@@ should optionally be on a per-buffer basis
    int l = getText().length();
    if( l == 0 ){
      try{
        // Insert the name and the semicolon at the beginning

        // @@@ It is arguable this shouldn't always be a semicolon,
        // because of how we are doing "sendee" parsing at a newline.

        String name = CalValue.get( "GLOBAL", value );
        String text = getText();

        setText( name + ';' + text );

        return true;
      } catch( CalUnknownValueException e ){
        // Don't do any processing.  No value.
        return false;
      }
    } else {
      // Do not do any processing on it, there is already data
      return false;
    }
  }

//---------------------------------------------------------------------------

  int findEndName(){
    // Try to treat the string as a name and a message, seperated by
    // a semicolon (@@@ use recip_regexp!).  The name cannot have
    // spaces.
    //
    // Return -1 if there is no name, otherwise return the index
    // of the ;
    
    int ret = -1;
    String text = getText();
    int firstspace = text.indexOf( ' ' );
    int firstsemi = text.indexOf( ';' );
    if( (firstsemi != -1) &&
        ( (firstsemi < firstspace) || (firstspace == -1) ) ){
        ret = firstsemi;
    }

    return ret;
  }
  
//---------------------------------------------------------------------------
  
  void replaceName( String name ){
    // Search for a name, if any, and replace it.  If there is no
    // name, just insert one.
    // Name should have trailing ;

    String text = getText();

    int semicolon = findEndName();
    try{
      text = text.substring( semicolon+1 );
    } catch( StringIndexOutOfBoundsException sioobe ){
      // Will fall here if it didn't find the semicolon, which is fine
    }

    setText( name + ';' + text );
  }

//---------------------------------------------------------------------------

  void scrollMsg( int dir ){

    System.out.println( "scrollMsg " + dir );

    String newtext = "**error**";
  
    if( sent_msg_ptr == -1 ){
      sent_msg_save = getText();
    }

    sent_msg_ptr += dir;

    if( sent_msg_ptr == -2 ){
      // Looped bakcwards
      sent_msg_ptr = sent_msg.size() - 1;
    }

    try{
      newtext = (String)sent_msg.elementAt( sent_msg_ptr );
    } catch( ArrayIndexOutOfBoundsException except ){
      newtext = sent_msg_save;
      sent_msg_ptr = -1;
    }

    setText( newtext );
  }

//---------------------------------------------------------------------------

  void handleEnter(){

    // Get the value in the entry area
    String text = getText();

    // Remove any newlines that might be hiding in here
    text = CalUtil.remove( text, "\r\n" );

    // Clear the line counter if we're not maxed out
    // Do this before we add the text to the buffer so the message
    //   itself can't max out the buffer
    if( !lines.isMaxed() ){
      lines.clear();
    }

    // Send it to the network and the return buffer
    tonet.put( text + '\n' );
    fromnet.put( text + '\n' );

    // Try to determine who we just sent it to
    // @@@ Should be using recip-regexp here!
    int firstsemi = findEndName();
    if( firstsemi != -1 ){
      String last_send = text.substring( 0, firstsemi );

      // Set this as our last send
      CalValue.put( "GLOBAL", "last_send", last_send );
       // Put it in the list of people we've sent to
      sent_to.insertElementAt( last_send, 0 );
    }

    // Store the message
    sent_msg.insertElementAt( text, 0 );

    // We might have been using the sent lists, so reset them to the beginning
    sent_to_ptr = -1;
    sent_msg_ptr = -1;

    // Clear the entry
    setText( "" );
  }

//---------------------------------------------------------------------------

  public boolean handleEvent( Event e ){
    //System.out.println( "Mod: " + e.modifiers );
    //System.out.println( "key: " + e.key + " F12: " + Event.F12 );
    //System.out.println( "id: " +e.id+ " KAR: " + Event.KEY_ACTION_RELEASE );

    if( e.id == Event.KEY_PRESS ){
      if( e.key == ';' ){
        return insertName( "last_send" );
      } else if( e.key == ':' ){
        return insertName( "last_recip" );
      } else if( e.key == '=' ){
        return insertName( "last_group" );
      } else if( e.key == '\t' ){
        if( e.shiftDown() ){
          // Shifted.  Go backwards
          sent_to_ptr--;
          if( sent_to_ptr == -2 ){
            // Looped backwards
            sent_to_ptr = sent_to.size() - 1;
          }
        } else {
          // No shift.  Advance
          sent_to_ptr++;
        }
        try{
          String name = (String)sent_to.elementAt( sent_to_ptr );
          replaceName( name );
        } catch( ArrayIndexOutOfBoundsException except ){
          replaceName( "" );
          sent_to_ptr = -1;
        }
        return true;
      }

    } else if( e.id == Event.KEY_RELEASE ){
      // Normal keys?
      if( e.key == '\n' ){
        handleEnter();
        return true;
      } else if( e.metaDown() && (e.key == 'm') ){
        // @@@ This is dorked too
        // Trigger the MORE button
        lines.clear();
      
        // Get out
        return true;
      }
    } else if( e.id == Event.KEY_ACTION_RELEASE ){
      // Function keys?
      if( e.key == Event.F12 ){
        // @@@ done because the above is dorked
        // Trigger the MORE button
        lines.clear();
      
        // Get out
        return true;
      } else if( e.key == Event.UP ){
        // Move to the last message in the list
        scrollMsg( 1 );
      } else if( e.key == Event.DOWN ){
        // Move to the next message in the list
        scrollMsg( -1 );
      } else if( e.key == Event.PGUP ){
        // Move the scroll bar up
        area.pgup();
      } else if( e.key == Event.PGDN ){
        // Move the scroll bar down
        area.pgdn();
      }

      // Cannot get it to recognize "normal" keys in this section,
      // they are swallowed up by the text entry area.

    }
    return super.handleEvent( e );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

class LineCounter{

  String buffname;
  int current;
  int max;

//---------------------------------------------------------------------------

  LineCounter( String n, int m ){
    buffname = n;
    setMax( m );
    clear();
  }

//---------------------------------------------------------------------------

  public void clear(){
    current = 0;
  }

//---------------------------------------------------------------------------

  public void inc(){
    current++;
  }

//---------------------------------------------------------------------------

  public boolean isMaxed(){
    boolean enabled;
    try{
      enabled = CalValue.getBool( buffname, "pause" );
    } catch( CalUnknownValueException cuve ){
      enabled = true;
    }

    if( enabled ){
      return( current >= max );
    } else {
      return false;
    }
  }

//---------------------------------------------------------------------------

  public void setMax( int m ){
    max = m;
  }

//---------------------------------------------------------------------------

  public int getMax(){
    return max;
  }

//---------------------------------------------------------------------------

  public void enable( boolean status ){
    CalValue.putBool( buffname, "pause", status );
  }

//---------------------------------------------------------------------------

  public void enable(){
    enable( true );
  }

//---------------------------------------------------------------------------

  public void disable(){
    enable( false );
  }

//---------------------------------------------------------------------------

}

//***************************************************************************

